@@ -1,4 +1,5 @@
-# Code generation format strings for UFC (Unified Form-assembly Code) v. 2016.1.0.
+# -*- coding: utf-8 -*-
+# Code generation format strings for UFC (Unified Form-assembly Code) version 2016.2.0
 # This code is released into the public domain.
 #
 # The FEniCS Project (http://www.fenicsproject.org/) 2006-2016
@@ -28,6 +29,7 @@ public:
                        const double * coordinate_dofs,
                        int cell_orientation) const final override
   {
+%(tabulate_tensor_comment)s
 %(tabulate_tensor)s
   }
 
@@ -74,6 +76,7 @@ void %(classname)s::tabulate_tensor(double * A,
                                     const double * coordinate_dofs,
                                     int cell_orientation) const
 {
+%(tabulate_tensor_comment)s
 %(tabulate_tensor)s
 }
 """
@@ -104,6 +107,7 @@ public:
                        std::size_t facet,
                        int cell_orientation) const final override
   {
+%(tabulate_tensor_comment)s
 %(tabulate_tensor)s
   }
 
@@ -152,6 +156,7 @@ void %(classname)s::tabulate_tensor(double * A,
                                     std::size_t facet,
                                     int cell_orientation) const
 {
+%(tabulate_tensor_comment)s
 %(tabulate_tensor)s
 }
 """
@@ -185,6 +190,7 @@ public:
                        int cell_orientation_0,
                        int cell_orientation_1) const final override
   {
+%(tabulate_tensor_comment)s
 %(tabulate_tensor)s
   }
 
@@ -239,6 +245,7 @@ void %(classname)s::tabulate_tensor(double * A,
                                     int cell_orientation_0,
                                     int cell_orientation_1) const
 {
+%(tabulate_tensor_comment)s
 %(tabulate_tensor)s
 }
 """
@@ -269,6 +276,7 @@ public:
                        std::size_t vertex,
                        int cell_orientation) const final override
   {
+%(tabulate_tensor_comment)s
 %(tabulate_tensor)s
   }
 
@@ -317,6 +325,7 @@ void %(classname)s::tabulate_tensor(double * A,
                                     std::size_t vertex,
                                     int cell_orientation) const
 {
+%(tabulate_tensor_comment)s
 %(tabulate_tensor)s
 }
 """
@@ -355,6 +364,7 @@ public:
                        const double * facet_normals,
                        int cell_orientation) const final override
   {
+%(tabulate_tensor_comment)s
 %(tabulate_tensor)s
   }
 
@@ -416,6 +426,7 @@ void %(classname)s::tabulate_tensor(double * A,
                                     const double * facet_normals,
                                     int cell_orientation) const
 {
+%(tabulate_tensor_comment)s
 %(tabulate_tensor)s
 }
 """
@@ -448,6 +459,7 @@ public:
                        const double * quadrature_weights,
                        int cell_orientation) const final override
   {
+%(tabulate_tensor_comment)s
 %(tabulate_tensor)s
   }
 
@@ -500,6 +512,7 @@ void %(classname)s::tabulate_tensor(double * A,
                                     const double * quadrature_weights,
                                     int cell_orientation) const
 {
+%(tabulate_tensor_comment)s
 %(tabulate_tensor)s
 }
 """
@@ -533,6 +546,7 @@ public:
                        const double * facet_normals,
                        int cell_orientation) const final override
   {
+%(tabulate_tensor_comment)s
 %(tabulate_tensor)s
   }
 
@@ -587,6 +601,7 @@ void %(classname)s::tabulate_tensor(double * A,
                                     const double * facet_normals,
                                     int cell_orientation) const
 {
+%(tabulate_tensor_comment)s
 %(tabulate_tensor)s
 }
 """
@@ -619,6 +634,7 @@ public:
                        const double * quadrature_weights,
                        int cell_orientation) const final override
   {
+%(tabulate_tensor_comment)s
 %(tabulate_tensor)s
   }
 
@@ -671,6 +687,7 @@ void %(classname)s::tabulate_tensor(double * A,
                                     const double * quadrature_weights,
                                     int cell_orientation) const
 {
+%(tabulate_tensor_comment)s
 %(tabulate_tensor)s
 }
 """
