@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """
 Compiler stage 4: Code generation
 ---------------------------------
@@ -6,7 +7,8 @@ This module implements the generation of C++ code for the body of each
 UFC function from an (optimized) intermediate representation (OIR).
 """
 
-# Copyright (C) 2009-2015 Anders Logg
+# Copyright (C) 2009-2016 Anders Logg, Martin Sandve Alnæs, Marie E. Rognes,
+# Kristian B. Oelgaard, and others
 #
 # This file is part of FFC.
 #
@@ -22,14 +24,14 @@ UFC function from an (optimized) intermediate representation (OIR).
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
-#
-# Modified by Mehdi Nikbakht 2010
-# Modified by Martin Alnaes, 2013-2015
+
+from itertools import chain
+from ufl import product
 
 # FFC modules
-from ffc.log import info, begin, end, debug_code
+from ffc.log import info, begin, end, debug_code, dstr
 from ffc.cpp import format, indent
-from ffc.cpp import set_exception_handling
+from ffc.cpp import set_exception_handling, set_float_formatting
 
 # FFC code generation modules
 from ffc.evaluatebasis import _evaluate_basis, _evaluate_basis_all
@@ -37,9 +39,9 @@ from ffc.evaluatebasisderivatives import _evaluate_basis_derivatives
 from ffc.evaluatebasisderivatives import _evaluate_basis_derivatives_all
 from ffc.evaluatedof import evaluate_dof_and_dofs, affine_weights
 from ffc.interpolatevertexvalues import interpolate_vertex_values
-
 from ffc.representation import pick_representation, ufc_integral_types
 
+
 # Errors issued for non-implemented functions
 def _not_implemented(function_name, return_null=False):
     body = format["exception"]("%s not yet implemented." % function_name)
@@ -48,47 +50,52 @@ def _not_implemented(function_name, return_null=False):
     return body
 
 
-def generate_code(ir, prefix, parameters):
+def generate_code(ir, parameters):
     "Generate code from intermediate representation."
 
     begin("Compiler stage 4: Generating code")
 
-    # FIXME: Document option -fconvert_exceptions_to_warnings
-    # FIXME: Remove option epsilon and just rely on precision?
-
+    # FIXME: This has global side effects
     # Set code generation parameters
-#    set_float_formatting(int(parameters["precision"]))
+    set_float_formatting(int(parameters["precision"]))
     set_exception_handling(parameters["convert_exceptions_to_warnings"])
 
     # Extract representations
-    ir_elements, ir_dofmaps, ir_integrals, ir_forms = ir
+    ir_elements, ir_dofmaps, ir_coordinate_mappings, ir_integrals, ir_forms = ir
 
     # Generate code for elements
     info("Generating code for %d element(s)" % len(ir_elements))
-    code_elements = [_generate_element_code(ir, prefix, parameters)
+    code_elements = [_generate_element_code(ir, parameters)
                      for ir in ir_elements]
 
     # Generate code for dofmaps
     info("Generating code for %d dofmap(s)" % len(ir_dofmaps))
-    code_dofmaps = [_generate_dofmap_code(ir, prefix, parameters)
+    code_dofmaps = [_generate_dofmap_code(ir, parameters)
                     for ir in ir_dofmaps]
 
+    # Generate code for coordinate_mappings
+    info("Generating code for %d coordinate_mapping(s)" % len(ir_coordinate_mappings))
+    code_coordinate_mappings = [_generate_coordinate_mapping_code(ir, parameters)
+                                for ir in ir_coordinate_mappings]
+    # FIXME: This disables output of generated coordinate_mapping class, until implemented properly
+    code_coordinate_mappings = []
+
     # Generate code for integrals
     info("Generating code for integrals")
-    code_integrals = [_generate_integral_code(ir, prefix, parameters)
+    code_integrals = [_generate_integral_code(ir, parameters)
                       for ir in ir_integrals]
 
     # Generate code for forms
     info("Generating code for forms")
-    code_forms = [_generate_form_code(ir, prefix, parameters)
+    code_forms = [_generate_form_code(ir, parameters)
                   for ir in ir_forms]
 
     end()
 
-    return code_elements, code_dofmaps, code_integrals, code_forms
+    return code_elements, code_dofmaps, code_coordinate_mappings, code_integrals, code_forms
 
 
-def _generate_element_code(ir, prefix, parameters):
+def _generate_element_code(ir, parameters):
     "Generate code for finite element from intermediate representation."
 
     # Skip code generation if ir is None
@@ -97,7 +104,6 @@ def _generate_element_code(ir, prefix, parameters):
 
     # Prefetch formatting to speedup code generation
     ret = format["return"]
-    classname = format["classname finite_element"]
     do_nothing = format["do nothing"]
     create = format["create foo"]
 
@@ -105,37 +111,50 @@ def _generate_element_code(ir, prefix, parameters):
     (evaluate_dof_code, evaluate_dofs_code) \
         = evaluate_dof_and_dofs(ir["evaluate_dof"])
 
+    #element_number = ir["id"]
+
     # Generate code
     code = {}
-    code["classname"] = classname(prefix, ir["id"])
+    code["classname"] = ir["classname"]
     code["members"] = ""
     code["constructor"] = do_nothing
     code["constructor_arguments"] = ""
     code["initializer_list"] = ""
     code["destructor"] = do_nothing
+
     code["signature"] = ret('"%s"' % ir["signature"])
     code["cell_shape"] = ret(format["cell"](ir["cell_shape"]))
     code["topological_dimension"] = ret(ir["topological_dimension"])
     code["geometric_dimension"] = ret(ir["geometric_dimension"])
     code["space_dimension"] = ret(ir["space_dimension"])
-    code["value_rank"] = ret(ir["value_rank"])
-    code["value_dimension"] = _value_dimension(ir["value_dimension"])
+
+    code["value_rank"] = ret(len(ir["value_shape"]))
+    code["value_dimension"] = _value_dimension(ir["value_shape"])
+    code["value_size"] = ret(product(ir["value_shape"]))
+    code["reference_value_rank"] = ret(len(ir["reference_value_shape"]))
+    code["reference_value_dimension"] = _value_dimension(ir["reference_value_shape"])
+    code["reference_value_size"] = ret(product(ir["reference_value_shape"]))
+
+    code["degree"] = ret(ir["degree"])
+    code["family"] = ret('"%s"' % (ir["family"],))
+
     code["evaluate_basis"] = _evaluate_basis(ir["evaluate_basis"])
     code["evaluate_basis_all"] = _evaluate_basis_all(ir["evaluate_basis"])
     code["evaluate_basis_derivatives"] \
         = _evaluate_basis_derivatives(ir["evaluate_basis"])
     code["evaluate_basis_derivatives_all"] \
         = _evaluate_basis_derivatives_all(ir["evaluate_basis"])
+
     code["evaluate_dof"] = evaluate_dof_code
     code["evaluate_dofs"] = evaluate_dofs_code
     code["interpolate_vertex_values"] \
         = interpolate_vertex_values(ir["interpolate_vertex_values"])
-    code["map_from_reference_cell"] \
-        = _not_implemented("map_from_reference_cell")
-    code["map_to_reference_cell"] = _not_implemented("map_to_reference_cell")
+    code["tabulate_dof_coordinates"] \
+        = _tabulate_dof_coordinates(ir["tabulate_dof_coordinates"])
     code["num_sub_elements"] = ret(ir["num_sub_elements"])
-    code["create_sub_element"] = _create_sub_element(prefix, ir)
+    code["create_sub_element"] = _create_sub_element(ir)
     code["create"] = ret(create(code["classname"]))
+    code["additional_includes_set"] = _additional_includes_finite_element(ir)
 
     # Postprocess code
     _postprocess_code(code, parameters)
@@ -143,7 +162,7 @@ def _generate_element_code(ir, prefix, parameters):
     return code
 
 
-def _generate_dofmap_code(ir, prefix, parameters):
+def _generate_dofmap_code(ir, parameters):
     "Generate code for dofmap from intermediate representation."
 
     # Skip code generation if ir is None
@@ -152,7 +171,6 @@ def _generate_dofmap_code(ir, prefix, parameters):
 
     # Prefetch formatting to speedup code generation
     ret = format["return"]
-    classname = format["classname dofmap"]
     declare = format["declaration"]
     assign = format["assign"]
     do_nothing = format["do nothing"]
@@ -161,9 +179,11 @@ def _generate_dofmap_code(ir, prefix, parameters):
     f_d = format["argument dimension"]
     create = format["create foo"]
 
+    #element_number = ir["id"]
+
     # Generate code
     code = {}
-    code["classname"] = classname(prefix, ir["id"])
+    code["classname"] = ir["classname"]
     code["members"] = ""
     code["constructor"] = do_nothing
     code["constructor_arguments"] = ""
@@ -173,30 +193,123 @@ def _generate_dofmap_code(ir, prefix, parameters):
     code["needs_mesh_entities"] \
         = _needs_mesh_entities(ir["needs_mesh_entities"])
     code["topological_dimension"] = ret(ir["topological_dimension"])
-    code["geometric_dimension"] = ret(ir["geometric_dimension"])
     code["global_dimension"] = _global_dimension(ir["global_dimension"])
     code["num_element_dofs"] = ret(ir["num_element_dofs"])
     code["num_facet_dofs"] = ret(ir["num_facet_dofs"])
     code["num_entity_dofs"] \
         = switch(f_d, [ret(num) for num in ir["num_entity_dofs"]],
                  ret(f_int(0)))
+    code["num_entity_closure_dofs"] \
+        = switch(f_d, [ret(num) for num in ir["num_entity_closure_dofs"]],
+                 ret(f_int(0)))
     code["tabulate_dofs"] = _tabulate_dofs(ir["tabulate_dofs"])
     code["tabulate_facet_dofs"] \
         = _tabulate_facet_dofs(ir["tabulate_facet_dofs"])
     code["tabulate_entity_dofs"] \
         = _tabulate_entity_dofs(ir["tabulate_entity_dofs"])
-    code["tabulate_coordinates"] \
-        = _tabulate_coordinates(ir["tabulate_coordinates"])
+    code["tabulate_entity_closure_dofs"] \
+        = _tabulate_entity_closure_dofs(ir["tabulate_entity_closure_dofs"])
     code["num_sub_dofmaps"] = ret(ir["num_sub_dofmaps"])
-    code["create_sub_dofmap"] = _create_sub_dofmap(prefix, ir)
+    code["create_sub_dofmap"] = _create_sub_dofmap(ir)
     code["create"] = ret(create(code["classname"]))
+    code["additional_includes_set"] = _additional_includes_dofmap(ir)
 
     # Postprocess code
     _postprocess_code(code, parameters)
 
     return code
 
-def _generate_integral_code(ir, prefix, parameters):
+
+def _additional_includes_dofmap(ir):
+    if not ir["jit"]:
+        return set()
+    dofmap_classnames = ir["create_sub_dofmap"]
+    jit_includes = [classname.split("_dofmap")[0] + ".h"
+                    for classname in dofmap_classnames]
+    return set("#include <%s>" % inc for inc in jit_includes)
+
+
+def _additional_includes_finite_element(ir):
+    if not ir["jit"]:
+        return set()
+    finite_element_classnames = ir["create_sub_element"]
+    jit_includes = [classname.split("_finite_element")[0] + ".h"
+                    for classname in finite_element_classnames]
+    return set("#include <%s>" % inc for inc in jit_includes)
+
+
+def _additional_includes_coordinate_mapping(ir):
+    if not ir["jit"]:
+        return set()
+    finite_element_classnames = [
+        ir["coordinate_finite_element_classname"],
+        ir["scalar_coordinate_finite_element_classname"]
+        ]
+    jit_includes = [classname.split("_finite_element")[0] + ".h"
+                    for classname in finite_element_classnames]
+    return set("#include <%s>" % inc for inc in jit_includes)
+
+
+def _additional_includes_form(ir):
+    if not ir["jit"]:
+        return set()
+    # Gather all header names for classes that are separately compiled
+    # For finite_element and dofmap the module and header name is the prefix,
+    # extracted here with .split, and equal for both classes so we skip dofmap here:
+    finite_element_classnames = list(chain(
+        ir["create_finite_element"],
+        ir["create_coordinate_finite_element"]
+        ))
+    jit_includes = set(classname.split("_finite_element")[0] + ".h"
+                       for classname in finite_element_classnames)
+    # FIXME: Enable when coordinate_mapping is fully generated:
+    #jit_includes.update(classname + ".h" for classname in ir["create_coordinate_mapping"])
+    return set("#include <%s>" % inc for inc in jit_includes)
+
+
+def _generate_coordinate_mapping_code(ir, parameters):
+    "Generate code for coordinate_mapping from intermediate representation."
+
+    # Skip code generation if ir is None
+    if ir is None:
+        return None
+
+    coordinate_mapping_number = ir["id"]
+
+    # FIXME: Get code dict from current work in uflacs
+
+    # Generate code
+    code = {}
+    code["classname"] = ir["classname"]
+
+    code["members"] = ""
+    code["constructor"] = ""
+    code["constructor_arguments"] = ""
+    code["initializer_list"] = ""
+    code["destructor"] = ""
+    code["create"] = "return nullptr;"
+
+    code["signature"] = 'return "fixme";'
+    code["cell_shape"] = "return ufc::shape::triangle;"
+    code["topological_dimension"] = "return 2;"
+    code["geometric_dimension"] = "return 3;"
+
+    code["create_coordinate_finite_element"] = "return nullptr;"
+    code["create_coordinate_dofmap"] = "return nullptr;"
+
+    code["compute_physical_coordinates"] = ""
+    code["compute_reference_coordinates"] = ""
+    code["compute_jacobians"] = ""
+    code["compute_jacobian_determinants"] = ""
+    code["compute_jacobian_inverses"] = ""
+    code["compute_geometry"] = ""
+
+    code["additional_includes_set"] = _additional_includes_coordinate_mapping(ir)
+
+    return code
+
+
+def _generate_integral_code(ir, parameters):
     "Generate code for integrals from intermediate representation."
 
     # Skip code generation if ir is None
@@ -207,16 +320,44 @@ def _generate_integral_code(ir, prefix, parameters):
     r = pick_representation(ir["representation"])
 
     # Generate code
-    code = r.generate_integral_code(ir, prefix, parameters)
+    # TODO: Drop prefix argument and get from ir:
+    code = r.generate_integral_code(ir, ir["prefix"], parameters)
+
+    # Generate comment
+    code["tabulate_tensor_comment"] = _generate_tabulate_tensor_comment(ir, parameters)
 
     # Indent code (unused variables should already be removed)
     # FIXME: Remove this quick hack
     if ir["representation"] != "uflacs":
         _indent_code(code)
+    else:
+        code["tabulate_tensor_comment"] = indent(code["tabulate_tensor_comment"], 4)
 
     return code
 
 
+def _generate_tabulate_tensor_comment(ir, parameters):
+    "Generate comment for tabulate_tensor."
+
+    r = ir["representation"]
+    integrals_metadata = ir["integrals_metadata"]
+    integral_metadata = ir["integral_metadata"]
+
+    comment  = format["comment"]("This function was generated using '%s' representation" % r) + "\n"
+    comment += format["comment"]("with the following integrals metadata:") + "\n"
+    comment += format["comment"]("") + "\n"
+    comment += "\n".join([format["comment"]("  " + l) for l in dstr(integrals_metadata).split("\n")][:-1])
+    comment += "\n"
+    for i, metadata in enumerate(integral_metadata):
+        comment += format["comment"]("") + "\n"
+        comment += format["comment"]("and the following integral %d metadata:" % i) + "\n"
+        comment += format["comment"]("") + "\n"
+        comment += "\n".join([format["comment"]("  " + l) for l in dstr(metadata).split("\n")][:-1])
+        comment += "\n"
+
+    return comment
+
+
 def _generate_original_coefficient_position(original_coefficient_positions):
     # TODO: I don't know how to implement this using the format dict,
     # this will do for now:
@@ -225,11 +366,11 @@ def _generate_original_coefficient_position(original_coefficient_positions):
     code = '\n'.join([
         "static const std::vector<std::size_t> position({%s});"
         % initializer_list, "return position[i];",
-        ])
+    ])
     return code
 
 
-def _generate_form_code(ir, prefix, parameters):
+def _generate_form_code(ir, parameters):
     "Generate code for form from intermediate representation."
 
     # Skip code generation if ir is None
@@ -238,12 +379,11 @@ def _generate_form_code(ir, prefix, parameters):
 
     # Prefetch formatting to speedup code generation
     ret = format["return"]
-    classname = format["classname form"]
     do_nothing = format["do nothing"]
 
     # Generate code
     code = {}
-    code["classname"] = classname(prefix, ir["id"])
+    code["classname"] = ir["classname"]
     code["members"] = ""
 
     code["constructor"] = do_nothing
@@ -252,18 +392,29 @@ def _generate_form_code(ir, prefix, parameters):
     code["destructor"] = do_nothing
 
     code["signature"] = ret('"%s"' % ir["signature"])
-    code["original_coefficient_position"] = _generate_original_coefficient_position(ir["original_coefficient_positions"])
+    code["original_coefficient_position"] = \
+        _generate_original_coefficient_position(ir["original_coefficient_position"])
     code["rank"] = ret(ir["rank"])
     code["num_coefficients"] = ret(ir["num_coefficients"])
 
-    code["create_finite_element"] = _create_finite_element(prefix, ir)
-    code["create_dofmap"] = _create_dofmap(prefix, ir)
+    code["create_coordinate_finite_element"] = _create_coordinate_finite_element(ir)
+    code["create_coordinate_dofmap"] = _create_coordinate_dofmap(ir)
+    code["create_coordinate_mapping"] = _create_coordinate_mapping(ir)
+
+    code["create_finite_element"] = _create_finite_element(ir)
+    code["create_dofmap"] = _create_dofmap(ir)
+
+    code["additional_includes_set"] = _additional_includes_form(ir)
 
     for integral_type in ufc_integral_types:
-        code["max_%s_subdomain_id" % integral_type] = ret(ir["max_%s_subdomain_id" % integral_type])
-        code["has_%s_integrals" % integral_type] = _has_foo_integrals(ir, integral_type)
-        code["create_%s_integral" % integral_type] = _create_foo_integral(ir, integral_type, prefix)
-        code["create_default_%s_integral" % integral_type] = _create_default_foo_integral(ir, integral_type, prefix)
+        code["max_%s_subdomain_id" % integral_type] = \
+            ret(ir["max_%s_subdomain_id" % integral_type])
+        code["has_%s_integrals" % integral_type] = \
+            _has_foo_integrals(ir, integral_type)
+        code["create_%s_integral" % integral_type] = \
+            _create_foo_integral(ir, integral_type)
+        code["create_default_%s_integral" % integral_type] = \
+            _create_default_foo_integral(ir, integral_type)
 
     # Postprocess code
     _postprocess_code(code, parameters)
@@ -272,6 +423,7 @@ def _generate_form_code(ir, prefix, parameters):
 
 #--- Code generation for non-trivial functions ---
 
+
 def _value_dimension(ir):
     "Generate code for value_dimension."
     ret = format["return"]
@@ -349,8 +501,7 @@ def _tabulate_dofs(ir):
         return assign(component(dofs_variable, 0), 0)
 
     # Extract representation
-    (dofs_per_element, num_dofs_per_element, num_entities,
-     need_offset, fakes) = ir
+    (dofs_per_element, num_dofs_per_element, need_offset, fakes) = ir
 
     # Declare offset if needed
     code = []
@@ -382,7 +533,7 @@ def _tabulate_dofs(ir):
                 v = multiply([len(num[k]), component(entity_index, (dim, k))])
                 for (j, dof) in enumerate(dofs):
                     value = add([offset_name, v, j])
-                    code.append(assign(component(dofs_variable, dof+i), value))
+                    code.append(assign(component(dofs_variable, dof + i), value))
 
             # Update offset corresponding to mesh entity:
             if need_offset:
@@ -395,12 +546,12 @@ def _tabulate_dofs(ir):
     return "\n".join(code)
 
 
-def _tabulate_coordinates(ir):
-    "Generate code for tabulate_coordinates."
+def _tabulate_dof_coordinates(ir):
+    "Generate code for tabulate_dof_coordinates."
 
-    # Raise error if tabulate_coordinates is ill-defined
+    # Raise error if tabulate_dof_coordinates is ill-defined
     if not ir:
-        msg = "tabulate_coordinates is not defined for this element"
+        msg = "tabulate_dof_coordinates is not defined for this element"
         return format["exception"](msg)
 
     # Extract formats:
@@ -408,7 +559,7 @@ def _tabulate_coordinates(ir):
     component = format["component"]
     precision = format["float"]
     assign = format["assign"]
-    f_x = format["vertex_coordinates"]
+    f_x = format["coordinate_dofs"]
     coordinates = format["argument coordinates"]
 
     # Extract coordinates and cell dimension
@@ -425,11 +576,11 @@ def _tabulate_coordinates(ir):
         w = coefficients(coordinate)
         for j in range(gdim):
             # Compute physical coordinate
-            coords = [component(f_x(), (k*gdim + j,)) for k in range(tdim + 1)]
+            coords = [component(f_x(), (k * gdim + j,)) for k in range(tdim + 1)]
             value = inner_product(w, coords)
 
             # Assign coordinate
-            code.append(assign(component(coordinates, (i*gdim + j)), value))
+            code.append(assign(component(coordinates, (i * gdim + j)), value))
 
     return "\n".join(code)
 
@@ -451,7 +602,7 @@ def _tabulate_entity_dofs(ir):
     dim = len(num_dofs_per_entity)
     excpt = format["exception"]("%s is larger than dimension (%d)"
                                 % (f_d, dim - 1))
-    code = [format["if"]("%s > %d" % (f_d, dim-1), excpt)]
+    code = [format["if"]("%s > %d" % (f_d, dim - 1), excpt)]
 
     # Generate cases for each dimension:
     all_cases = ["" for d in range(dim)]
@@ -481,42 +632,123 @@ def _tabulate_entity_dofs(ir):
     return "\n".join(code)
 
 
+def _tabulate_entity_closure_dofs(ir):
+    "Generate code for tabulate_entity_closure_dofs."
+
+    # Extract variables from ir
+    entity_closure_dofs, entity_dofs, num_dofs_per_entity = ir
+
+    # Prefetch formats
+    assign = format["assign"]
+    component = format["component"]
+    f_d = format["argument dimension"]
+    f_i = format["argument entity"]
+    dofs = format["argument dofs"]
+
+    # Add check that dimension and number of mesh entities is valid
+    dim = len(num_dofs_per_entity)
+    excpt = format["exception"]("%s is larger than dimension (%d)"
+                                % (f_d, dim - 1))
+    code = [format["if"]("%s > %d" % (f_d, dim - 1), excpt)]
+
+    # Generate cases for each dimension:
+    all_cases = ["" for d in range(dim)]
+    for d in range(dim):
+        num_entities = len(entity_dofs[d])
+
+        # Add check that given entity is valid:
+        excpt = format["exception"]("%s is larger than number of entities (%d)"
+                                    % (f_i, num_entities - 1))
+        check = format["if"]("%s > %d" % (f_i, num_entities - 1), excpt)
+
+        # Generate cases for each mesh entity
+        cases = []
+        for entity in range(num_entities):
+            assignments = [assign(component(dofs, j), dof)
+                           for (j, dof) in enumerate(entity_closure_dofs[(d, entity)])]
+            cases.append("\n".join(assignments))
+
+        # Generate inner switch with preceding check
+        all_cases[d] = "\n".join([check, format["switch"](f_i, cases)])
+
+    # Generate outer switch
+    code.append(format["switch"](f_d, all_cases))
+
+    return "\n".join(code)
+
+
 #--- Utility functions ---
 
-
-def _create_foo(prefix, class_name, postfix, arg, numbers=None):
-    "Generate code for create_<foo>."
+def _create_switch(arg, classnames, factory=False):
+    "Generate code for create_<bar>(arg) returning new <classnames[arg]>."
     ret = format["return"]
-    create = format["create foo"]
-
-    class_names = ["%s_%s_%d" % (prefix.lower(), class_name, i)
-                   for i in postfix]
-    cases = [ret(create(name)) for name in class_names]
+    if factory:
+        create = format["create factory"]
+    else:
+        create = format["create foo"]
+    numbers = list(range(len(classnames)))
+    cases = [ret(create(name)) for name in classnames]
     default = ret(0)
     return format["switch"](arg, cases, default=default, numbers=numbers)
 
-def _create_finite_element(prefix, ir):
-    f_i = format["argument sub"]
-    return _create_foo(prefix, "finite_element", ir["create_finite_element"], f_i)
 
-def _create_dofmap(prefix, ir):
-    f_i = format["argument sub"]
-    return _create_foo(prefix, "dofmap", ir["create_dofmap"], f_i)
+def _create_coordinate_finite_element(ir):
+    ret = format["return"]
+    if ir["jit"]:
+        create = format["create factory"]
+    else:
+        create = format["create foo"]
+    classnames = ir["create_coordinate_finite_element"]
+    assert len(classnames) == 1  # list of length 1 until we support multiple domains
+    return ret(create(classnames[0]))
 
-def _create_sub_element(prefix, ir):
-    f_i = format["argument sub"]
-    return _create_foo(prefix, "finite_element", ir["create_sub_element"], f_i)
 
-def _create_sub_dofmap(prefix, ir):
+def _create_coordinate_dofmap(ir):
+    ret = format["return"]
+    if ir["jit"]:
+        create = format["create factory"]
+    else:
+        create = format["create foo"]
+    classnames = ir["create_coordinate_dofmap"]
+    assert len(classnames) == 1  # list of length 1 until we support multiple domains
+    return ret(create(classnames[0]))
+
+
+def _create_coordinate_mapping(ir):
+    ret = format["return"]
+    if ir["jit"]:
+        create = format["create factory"]
+    else:
+        create = format["create foo"]
+    classnames = ir["create_coordinate_mapping"]
+    assert len(classnames) == 1  # list of length 1 until we support multiple domains
+    # return ret(create(classnames[0]))
+    return ret("nullptr")  # FIXME: Disabled until we generate a functional class (work in progress)
+
+
+def _create_finite_element(ir):
     f_i = format["argument sub"]
-    return _create_foo(prefix, "dofmap", ir["create_sub_dofmap"], f_i)
+    classnames = ir["create_finite_element"]
+    return _create_switch(f_i, classnames, ir["jit"])
+
+
+def _create_dofmap(ir):
+    f_i = format["argument sub"]
+    classnames = ir["create_dofmap"]
+    return _create_switch(f_i, classnames, ir["jit"])
+
+
+def _create_sub_element(ir):
+    f_i = format["argument sub"]
+    classnames = ir["create_sub_element"]
+    return _create_switch(f_i, classnames, ir["jit"])
+
+
+def _create_sub_dofmap(ir):
+    f_i = format["argument sub"]
+    classnames = ir["create_sub_dofmap"]
+    return _create_switch(f_i, classnames, ir["jit"])
 
-def _create_foo_integral(ir, integral_type, prefix):
-    "Generate code for create_<foo>_integral."
-    f_i = format["argument subdomain"]
-    class_name = integral_type + "_integral_" + str(ir["id"])
-    postfix = ir["create_" + integral_type + "_integral"]
-    return _create_foo(prefix, class_name, postfix, f_i, numbers=postfix)
 
 def _has_foo_integrals(ir, integral_type):
     ret = format["return"]
@@ -524,17 +756,27 @@ def _has_foo_integrals(ir, integral_type):
     i = ir["has_%s_integrals" % integral_type]
     return ret(b(i))
 
-def _create_default_foo_integral(ir, integral_type, prefix):
+
+def _create_foo_integral(ir, integral_type):
+    "Generate code for create_<foo>_integral."
+    ret = format["return"]
+    create = format["create foo"]
+    f_i = format["argument subdomain"]
+    subdomain_ids, classnames = ir["create_%s_integral" % integral_type]
+    cases = [ret(create(name)) for name in classnames]
+    default = ret(0)
+    return format["switch"](f_i, cases, default=default, numbers=subdomain_ids)
+
+
+def _create_default_foo_integral(ir, integral_type):
     "Generate code for create_default_<foo>_integral."
     ret = format["return"]
-    postfix = ir["create_default_" + integral_type + "_integral"]
-    if postfix is None:
+    classname = ir["create_default_%s_integral" % integral_type]
+    if classname is None:
         return ret(0)
     else:
         create = format["create foo"]
-        class_name = integral_type + "_integral_" + str(ir["id"])
-        name = "%s_%s_%s" % (prefix.lower(), class_name, postfix)
-        return ret(create(name))
+        return ret(create(classname))
 
 
 def _postprocess_code(code, parameters):
@@ -546,9 +788,9 @@ def _postprocess_code(code, parameters):
 def _indent_code(code):
     "Indent code that should be indented."
     for key in code:
-        if not key in ("classname", "members", "constructor_arguments",
+        if key not in ("classname", "members", "constructor_arguments",
                        "initializer_list", "additional_includes_set",
-                       "restrict", "class_type"):
+                       "class_type"):
             code[key] = indent(code[key], 4)
 
 
