@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """
 This is the compiler, acting as the main interface for compilation
 of forms and breaking the compilation into several sequential stages.
@@ -89,7 +90,7 @@ The compiler stages are implemented by the following functions:
   format_code       (stage 5)
 """
 
-# Copyright (C) 2007-2015 Anders Logg
+# Copyright (C) 2007-2016 Anders Logg
 #
 # This file is part of FFC.
 #
@@ -109,7 +110,7 @@ The compiler stages are implemented by the following functions:
 # Modified by Kristian B. Oelgaard, 2010.
 # Modified by Dag Lindbo, 2008.
 # Modified by Garth N. Wells, 2009.
-# Modified by Martin Alnaes, 2013-2015
+# Modified by Martin Sandve Alnæs, 2013-2016
 
 __all__ = ["compile_form", "compile_element"]
 
@@ -118,141 +119,112 @@ from time import time
 import os
 
 # FFC modules
-from ffc.log import info, info_green, warning
-from ffc.parameters import default_parameters
-
-# FFC modules
-from ffc.analysis import analyze_forms, analyze_elements
+from ffc.log import info, info_green, warning, error
+from ffc.parameters import validate_parameters
+from ffc.analysis import analyze_ufl_objects
 from ffc.representation import compute_ir
 from ffc.optimization import optimize_ir
 from ffc.codegeneration import generate_code
-from ffc.formatting import format_code, write_code
+from ffc.formatting import format_code
 from ffc.wrappers import generate_wrapper_code
 
-def compile_form(forms, object_names=None, prefix="Form", parameters=None):
-    """This function generates UFC code for a given UFL form or list
-    of UFL forms."""
-
-    info("Compiling form %s\n" % prefix)
-
-    # Reset timing
-    cpu_time_0 = time()
-
-    # Check input arguments
-    forms = _check_forms(forms)
-    if not forms:
-        return
-    if prefix != os.path.basename(prefix):
-        prefix = os.path.basename(prefix)
-        warning("Invalid prefix, modified to {}.".format(prefix))
-    if object_names is None:
-        object_names = {}
-    parameters = _check_parameters(parameters)
-
-    # Stage 1: analysis
-    cpu_time = time()
-    analysis = analyze_forms(forms, parameters)
-    _print_timing(1, time() - cpu_time)
-
-    # Stage 2: intermediate representation
-    cpu_time = time()
-    ir = compute_ir(analysis, parameters)
-    _print_timing(2, time() - cpu_time)
-
-    # Stage 3: optimization
-    cpu_time = time()
-    oir = optimize_ir(ir, parameters)
-    _print_timing(3, time() - cpu_time)
-
-    # Stage 4: code generation
-    cpu_time = time()
-    code = generate_code(oir, prefix, parameters)
-    _print_timing(4, time() - cpu_time)
-
-    # Stage 4.1: generate wrappers
-    cpu_time = time()
-    wrapper_code = generate_wrapper_code(analysis, prefix, object_names, parameters)
-    _print_timing(4.1, time() - cpu_time)
-
-    # Stage 5: format code
-    cpu_time = time()
-    code_h, code_c = format_code(code, wrapper_code, prefix, parameters)
-    write_code(code_h, code_c, prefix, parameters) # FIXME: Don't write to file in this function (issue #72)
-    _print_timing(5, time() - cpu_time)
-
-    info_green("FFC finished in %g seconds.", time() - cpu_time_0)
-
-def compile_element(elements, prefix="Element", parameters=None):
-    """This function generates UFC code for a given UFL element or
-    list of UFL elements."""
-
-    info("Compiling element %s\n" % prefix)
-
-    # Reset timing
-    cpu_time_0 = time()
-
-    # Check input arguments
-    elements = _check_elements(elements)
-    if not elements:
-        return
-    parameters = _check_parameters(parameters)
-
-    # Stage 1: analysis
-    cpu_time = time()
-    analysis = analyze_elements(elements, parameters)
-    _print_timing(1, time() - cpu_time)
-
-    # Stage 2: intermediate representation
-    cpu_time = time()
-    ir = compute_ir(analysis, parameters)
-    _print_timing(2, time() - cpu_time)
-
-    # Stage 3: optimization
-    cpu_time = time()
-    oir = optimize_ir(ir, parameters)
-    _print_timing(3, time() - cpu_time)
-
-    # Stage 4: code generation
-    cpu_time = time()
-    code = generate_code(oir, prefix, parameters)
-    _print_timing(4, time() - cpu_time)
-
-    # Stage 4.1: generate wrappers
-    cpu_time = time()
-    object_names = {}
-    wrapper_code = generate_wrapper_code(analysis, prefix, object_names, parameters)
-    _print_timing(4.1, time() - cpu_time)
-
-    # Stage 5: format code
-    cpu_time = time()
-    code_h, code_c = format_code(code, wrapper_code, prefix, parameters)
-    write_code(code_h, code_c, prefix, parameters) # FIXME: Don't write to file in this function (issue #72)
-    _print_timing(5, time() - cpu_time)
-
-    info_green("FFC finished in %g seconds.", time() - cpu_time_0)
-
-def _check_forms(forms):
-    "Initial check of forms."
-    if not isinstance(forms, (list, tuple)):
-        forms = (forms,)
-    return forms
-
-def _check_elements(elements):
-    "Initial check of elements."
-    if not isinstance(elements, (list, tuple)):
-        elements = (elements,)
-    return elements
-
-def _check_parameters(parameters):
-    "Initial check of parameters."
-    if parameters is None:
-        parameters = default_parameters()
-    if "blas" in parameters:
-        warning("BLAS mode unavailable (will return in a future version).")
-    if "quadrature_points" in parameters:
-        warning("Option 'quadrature_points' has been replaced by 'quadrature_degree'.")
-    return parameters
 
 def _print_timing(stage, timing):
     "Print timing results."
     info("Compiler stage %s finished in %g seconds.\n" % (str(stage), timing))
+
+
+def compile_form(forms, object_names=None,
+                 prefix="Form", parameters=None, jit=False):
+    """This function generates UFC code for a given UFL form or list of UFL forms."""
+    return compile_ufl_objects(forms, "form", object_names,
+                               prefix, parameters, jit)
+
+
+def compile_element(elements, object_names=None,
+                    prefix="Element", parameters=None, jit=False):
+    """This function generates UFC code for a given UFL element or list of UFL elements."""
+    return compile_ufl_objects(elements, "element", object_names,
+                               prefix, parameters, jit)
+
+
+def compile_coordinate_mapping(elements, object_names=None,
+                               prefix="Mesh", parameters=None, jit=False):
+    """This function generates UFC code for a given UFL element or list of UFL elements."""
+    return compile_ufl_objects(elements, "coordinate_mapping", object_names,
+                               prefix, parameters, jit)
+
+
+def compile_ufl_objects(ufl_objects, kind, object_names=None,
+                        prefix=None, parameters=None, jit=False):
+    """This function generates UFC code for a given UFL form or list of UFL forms."""
+    info("Compiling %s %s\n" % (kind, prefix))
+
+    # Reset timing
+    cpu_time_0 = time()
+
+    # Note that jit will always pass validated parameters so 
+    # this is only for commandline and direct call from python
+    if not jit:
+        parameters = validate_parameters(parameters)
+
+    # Check input arguments
+    if not isinstance(ufl_objects, (list, tuple)):
+        ufl_objects = (ufl_objects,)
+    if not ufl_objects:
+        return "", ""
+    if prefix != os.path.basename(prefix):
+        error("Invalid prefix, looks like a full path? prefix='{}'.".format(prefix))
+    if object_names is None:
+        object_names = {}
+
+    # Stage 1: analysis
+    cpu_time = time()
+    analysis = analyze_ufl_objects(ufl_objects, kind, parameters)
+    _print_timing(1, time() - cpu_time)
+
+    # Stage 2: intermediate representation
+    cpu_time = time()
+    ir = compute_ir(analysis, prefix, parameters, jit)
+    _print_timing(2, time() - cpu_time)
+
+    # Stage 3: optimization
+    cpu_time = time()
+    oir = optimize_ir(ir, parameters)
+    _print_timing(3, time() - cpu_time)
+
+    # Stage 4: code generation
+    cpu_time = time()
+    code = generate_code(oir, parameters)
+    _print_timing(4, time() - cpu_time)
+
+    # Stage 4.1: generate wrappers
+    cpu_time = time()
+    wrapper_code = generate_wrapper_code(analysis, prefix, object_names, parameters)
+    _print_timing(4.1, time() - cpu_time)
+
+    # Stage 5: format code
+    cpu_time = time()
+    code_h, code_c = format_code(code, wrapper_code, prefix, parameters, jit)
+    _print_timing(5, time() - cpu_time)
+
+    info_green("FFC finished in %g seconds.", time() - cpu_time_0)
+
+    if jit:
+        # Must use processed elements from analysis here
+        form_datas, unique_elements, element_numbers, unique_coordinate_elements = analysis
+
+        # Avoid returning self as dependency for infinite recursion
+        unique_elements = list(element for element in unique_elements
+                               if element not in ufl_objects)
+
+        # FIXME: May get similar recursion issue with coordinate elements
+        # but currently not used all the way
+        dependent_ufl_objects = {
+            "element": tuple(unique_elements),
+            "coordinate_mapping": tuple(unique_coordinate_elements),
+        }
+
+        return code_h, code_c, dependent_ufl_objects
+    else:
+        return code_h, code_c
