@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright (C) 2010 Marie E. Rognes
 #
 # This file is part of FFC.
@@ -22,7 +23,9 @@ import numpy
 from .utils import pick_first
 from .mixedelement import _combine_entity_dofs, _num_components
 
+
 class EnrichedElement:
+
     "Create the space spanned by a list of ffc elements."
 
     def __init__(self, elements):
@@ -68,7 +71,7 @@ class EnrichedElement:
             # Insert element table into table
             for dtuple in etable.keys():
 
-                if not dtuple in table:
+                if dtuple not in table:
                     if num_components == 1:
                         table[dtuple] = numpy.zeros((self.space_dimension(), len(points)))
                     else:
