@@ -1,8 +1,6 @@
-"""Code generation for evaluation of finite element basis values. This
-module generates code which is more or less a C++ representation of
-the code found in FIAT."""
+# -*- coding: utf-8 -*-
 
-# Copyright (C) 2007-2010 Kristian B. Oelgaard
+# Copyright (C) 2007-2016 Kristian B. Oelgaard
 #
 # This file is part of FFC.
 #
@@ -19,13 +17,14 @@ the code found in FIAT."""
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
-# First added:  2007-04-04
-# Last changed: 2015-03-28
-#
 # Modified by Marie E. Rognes 2011
 # Modified by Anders Logg 2013
-# Modified by Lizao Li 2015
-#
+# Modified by Lizao Li 2015, 2016
+
+"""Code generation for evaluation of finite element basis values. This
+module generates code which is more or less a C++ representation of
+the code found in FIAT."""
+
 # MER: The original module generated code that was more or less a C++
 # representation of the code found in FIAT. I've modified this (for 2
 # and 3D) to generate code that does the same as FIAT, but with loops
@@ -35,33 +34,35 @@ the code found in FIAT."""
 # Python modules
 import math
 import numpy
+from six import string_types
 
 # FFC modules
 from ffc.log import error
 from ffc.cpp import remove_unused, indent, format
-from ffc.quadrature.symbolics import create_float, create_float, create_symbol,\
-                                     create_product, create_sum, create_fraction, CONST
+from ffc.quadrature.symbolics import create_float, create_symbol,\
+    create_product, create_sum, create_fraction, CONST
+
 
 def _evaluate_basis_all(data):
     """Like evaluate_basis, but return the values of all basis functions (dofs)."""
 
-    if isinstance(data, str):
+    if isinstance(data, string_types):
         return format["exception"]("evaluate_basis_all: %s" % data)
 
     # Prefetch formats.
-    f_assign    = format["assign"]
+    f_assign = format["assign"]
     f_component = format["component"]
-    f_comment   = format["comment"]
-    f_loop      = format["generate loop"]
-    f_r, f_s    = format["free indices"][:2]
-    f_tensor    = format["tabulate tensor"]
-    f_values    = format["argument values"]
-    f_basis     = format["call basis"]
-    f_dof_vals  = format["dof values"]
-    f_double    = format["float declaration"]
-    f_float     = format["floating point"]
-    f_decl      = format["declaration"]
-    f_ref_var   = format["reference variable"]
+    f_comment = format["comment"]
+    f_loop = format["generate loop"]
+    f_r, f_s = format["free indices"][:2]
+    f_tensor = format["tabulate tensor"]
+    f_values = format["argument values"]
+    f_basis = format["call basis"]
+    f_dof_vals = format["dof values"]
+    f_double = format["float declaration"]
+    f_float = format["floating point"]
+    f_decl = format["declaration"]
+    f_ref_var = format["reference variable"]
 
     # Initialise return code.
     code = []
@@ -94,7 +95,7 @@ def _evaluate_basis_all(data):
     else:
         code += [f_decl(f_double,
                         f_component(f_dof_vals, physical_value_size),
-                        f_tensor([0.0]*physical_value_size))]
+                        f_tensor([0.0] * physical_value_size))]
 
     # Create loop over dofs that calls evaluate_basis for a single dof and
     # inserts the values into the global array.
@@ -107,7 +108,7 @@ def _evaluate_basis_all(data):
     else:
         lines_r += [f_basis(f_r, f_dof_vals)]
 
-    if physical_value_size ==  1:
+    if physical_value_size == 1:
         lines_r += [f_assign(f_component(f_values, f_r), f_dof_vals)]
     else:
         index = format["matrix index"](f_r, f_s, physical_value_size)
@@ -120,6 +121,8 @@ def _evaluate_basis_all(data):
     return "\n".join(code)
 
 # From FIAT_NEW.polynomial_set.tabulate()
+
+
 def _evaluate_basis(data):
     """Generate run time code to evaluate an element basisfunction at an
     arbitrary point. The value(s) of the basisfunction is/are
@@ -130,14 +133,14 @@ def _evaluate_basis(data):
     The function should work for all elements supported by FIAT, but it remains
     untested for tensor valued elements."""
 
-    if isinstance(data, str):
+    if isinstance(data, string_types):
         return format["exception"]("evaluate_basis: %s" % data)
 
     # Prefetch formats.
-    f_assign    = format["assign"]
-    f_comment   = format["comment"]
-    f_values    = format["argument values"]
-    f_float     = format["floating point"]
+    f_assign = format["assign"]
+    f_comment = format["comment"]
+    f_values = format["argument values"]
+    f_float = format["floating point"]
     f_component = format["component"]
 
     # Initialise return code.
@@ -175,9 +178,10 @@ def _evaluate_basis(data):
 
     # Remove unused variables (from transformations and mappings) in code.
     code = remove_unused("\n".join(code))
-    #code = "\n".join(code)
+    # code = "\n".join(code)
     return code
 
+
 def _generate_dof_code(data, dof_data):
     """Generate code for a single basis element as the dot product of
     coefficients and basisvalues. Then apply transformation if applicable."""
@@ -194,18 +198,19 @@ def _generate_dof_code(data, dof_data):
 
     return remove_unused("\n".join(code))
 
+
 def _tabulate_coefficients(dof_data):
     """This function tabulates the element coefficients that are
     generated by FIAT at compile time."""
 
     # Prefetch formats to speed up code generation.
-    f_comment       = format["comment"]
-    f_table         = format["static const float declaration"]
-    f_coefficients  = format["coefficients"]
-    f_component     = format["component"]
-    f_decl          = format["declaration"]
-    f_tensor        = format["tabulate tensor"]
-    f_new_line      = format["new line"]
+    f_comment = format["comment"]
+    f_table = format["static const float declaration"]
+    f_coefficients = format["coefficients"]
+    f_component = format["component"]
+    f_decl = format["declaration"]
+    f_tensor = format["tabulate tensor"]
+    f_new_line = format["new line"]
 
     # Get coefficients from basis functions, computed by FIAT at compile time.
     coefficients = dof_data["coeffs"]
@@ -226,30 +231,31 @@ def _tabulate_coefficients(dof_data):
         code += [f_decl(f_table, name, f_new_line + f_tensor(coeffs))] + [""]
     return code
 
+
 def _compute_values(data, dof_data):
     """This function computes the value of the basisfunction as the dot product
     of the coefficients and basisvalues."""
 
     # Prefetch formats to speed up code generation.
-    f_values        = format["argument values"]
-    f_component     = format["component"]
-    f_comment       = format["comment"]
-    f_add           = format["add"]
-    f_coefficients  = format["coefficients"]
-    f_basisvalues   = format["basisvalues"]
-    f_r             = format["free indices"][0]
+    f_values = format["argument values"]
+    f_component = format["component"]
+    f_comment = format["comment"]
+    f_add = format["add"]
+    f_coefficients = format["coefficients"]
+    f_basisvalues = format["basisvalues"]
+    f_r = format["free indices"][0]
     f_deref_pointer = format["dereference pointer"]
-    f_detJ          = format["det(J)"]
-    f_inv           = format["inverse"]
-    f_mul           = format["mul"]
-    f_iadd          = format["iadd"]
-    f_group         = format["grouping"]
-    f_tmp_ref       = format["tmp ref value"]
-    f_assign        = format["assign"]
-    f_loop          = format["generate loop"]
-    f_const_float   = format["const float declaration"]
-    f_trans         = format["transform"]
-    f_inner         = format["inner product"]
+    f_detJ = format["det(J)"]
+    f_inv = format["inverse"]
+    f_mul = format["mul"]
+    f_iadd = format["iadd"]
+    f_group = format["grouping"]
+    f_tmp_ref = format["tmp ref value"]
+    f_assign = format["assign"]
+    f_loop = format["generate loop"]
+    f_const_float = format["const float declaration"]
+    f_trans = format["transform"]
+    f_inner = format["inner product"]
 
     tdim = data["topological_dimension"]
     gdim = data["geometric_dimension"]
@@ -261,7 +267,7 @@ def _compute_values(data, dof_data):
     num_components = dof_data["num_components"]
     reference_offset = dof_data["reference_offset"]
     physical_offset = dof_data["physical_offset"]
-    offset = reference_offset # physical_offset # FIXME: Should be physical offset but that breaks tests
+    offset = reference_offset  # physical_offset # FIXME: Should be physical offset but that breaks tests
 
     lines = []
     if data["reference_value_size"] != 1:
@@ -269,14 +275,14 @@ def _compute_values(data, dof_data):
         for i in range(num_components):
             # Generate name and value to create matrix vector multiply.
             name = f_component(f_values, i + offset)
-            value = f_mul([f_component(f_coefficients(i), f_r),\
-                    f_component(f_basisvalues, f_r)])
+            value = f_mul([f_component(f_coefficients(i), f_r),
+                           f_component(f_basisvalues, f_r)])
             lines += [f_iadd(name, value)]
     else:
         # Generate name and value to create matrix vector multiply.
         name = f_deref_pointer(f_values)
-        value = f_mul([f_component(f_coefficients(0), f_r),\
-                f_component(f_basisvalues, f_r)])
+        value = f_mul([f_component(f_coefficients(0), f_r),
+                       f_component(f_basisvalues, f_r)])
         lines = [f_iadd(name, value)]
 
     # Get number of members of the expansion set.
@@ -291,8 +297,8 @@ def _compute_values(data, dof_data):
     elif mapping == "contravariant piola":
         code += ["", f_comment("Using contravariant Piola transform to map values back to the physical element")]
         # Get temporary values before mapping.
-        code += [f_const_float(f_tmp_ref(i), f_component(f_values, i + offset))\
-                  for i in range(num_components)]
+        code += [f_const_float(f_tmp_ref(i), f_component(f_values, i + offset))
+                 for i in range(num_components)]
         # Create names for inner product.
         basis_col = [f_tmp_ref(j) for j in range(tdim)]
         for i in range(gdim):
@@ -307,8 +313,8 @@ def _compute_values(data, dof_data):
     elif mapping == "covariant piola":
         code += ["", f_comment("Using covariant Piola transform to map values back to the physical element")]
         # Get temporary values before mapping.
-        code += [f_const_float(f_tmp_ref(i), f_component(f_values, i + offset))\
-                  for i in range(num_components)]
+        code += [f_const_float(f_tmp_ref(i), f_component(f_values, i + offset))
+                 for i in range(num_components)]
         # Create names for inner product.
         tdim = data["topological_dimension"]
         gdim = data["geometric_dimension"]
@@ -321,11 +327,11 @@ def _compute_values(data, dof_data):
             value = f_group(f_inner(inv_jacobian_column, basis_col))
             name = f_component(f_values, i + offset)
             code += [f_assign(name, value)]
-    elif mapping == "pullback as metric":
-        code += ["", f_comment("Using metric pullback to map values back to the physical element")]
+    elif mapping == "double covariant piola":
+        code += ["", f_comment("Using double covariant Piola transform to map values back to the physical element")]
         # Get temporary values before mapping.
-        code += [f_const_float(f_tmp_ref(i), f_component(f_values, i + offset))\
-                  for i in range(num_components)]
+        code += [f_const_float(f_tmp_ref(i), f_component(f_values, i + offset))
+                 for i in range(num_components)]
         # Create names for inner product.
         tdim = data["topological_dimension"]
         gdim = data["geometric_dimension"]
@@ -344,57 +350,81 @@ def _compute_values(data, dof_data):
                  for k in range(tdim)]))
             name = f_component(f_values, p + offset)
             code += [f_assign(name, value)]
+    elif mapping == "double contravariant piola":
+        code += ["", f_comment("Using double contravariant Piola transform to map values back to the physical element")]
+        # Get temporary values before mapping.
+        code += [f_const_float(f_tmp_ref(i), f_component(f_values, i + offset))\
+                  for i in range(num_components)]
+        # Create names for inner product.
+        tdim = data["topological_dimension"]
+        gdim = data["geometric_dimension"]
+        basis_col = [f_tmp_ref(j) for j in range(num_components)]
+        for p in range(num_components):
+            # unflatten the indices
+            i = p // tdim
+            l = p % tdim
+            # g_il = (det J)^(-2) Jij G_jk Jlk
+            value = f_group(f_inner(
+                [f_inner([f_trans("J", i, j, tdim, gdim, None)
+                          for j in range(tdim)],
+                         [basis_col[j * tdim + k] for j in range(tdim)])
+                 for k in range(tdim)],
+                [f_trans("J", l, k, tdim, gdim, None) for k in range(tdim)]))
+            value = f_mul([f_inv(f_detJ(None)), f_inv(f_detJ(None)), value])
+            name = f_component(f_values, p + offset)
+            code += [f_assign(name, value)]
     else:
         error("Unknown mapping: %s" % mapping)
 
     return code
 
+
 def _compute_basisvalues(data, dof_data):
     """From FIAT_NEW.expansions."""
 
     UNROLL = True
 
     # Prefetch formats to speed up code generation.
-    f_comment     = format["comment"]
-    f_add         = format["add"]
-    f_mul         = format["mul"]
-    f_imul        = format["imul"]
-    f_sub         = format["sub"]
-    f_group       = format["grouping"]
-    f_assign      = format["assign"]
-    f_sqrt        = format["sqrt"]
-    f_x           = format["x coordinate"]
-    f_y           = format["y coordinate"]
-    f_z           = format["z coordinate"]
-    f_double      = format["float declaration"]
-    f_basisvalue  = format["basisvalues"]
-    f_component   = format["component"]
-    f_float       = format["floating point"]
-    f_uint        = format["uint declaration"]
-    f_tensor      = format["tabulate tensor"]
-    f_loop        = format["generate loop"]
-    f_decl        = format["declaration"]
-    f_tmp         = format["tmp value"]
-    f_int         = format["int"]
+    f_comment = format["comment"]
+    f_add = format["add"]
+    f_mul = format["mul"]
+    f_imul = format["imul"]
+    f_sub = format["sub"]
+    f_group = format["grouping"]
+    f_assign = format["assign"]
+    f_sqrt = format["sqrt"]
+    f_x = format["x coordinate"]
+    f_y = format["y coordinate"]
+    f_z = format["z coordinate"]
+    f_double = format["float declaration"]
+    f_basisvalue = format["basisvalues"]
+    f_component = format["component"]
+    f_float = format["floating point"]
+    f_uint = format["uint declaration"]
+    f_tensor = format["tabulate tensor"]
+    f_loop = format["generate loop"]
+    f_decl = format["declaration"]
+    f_tmp = format["tmp value"]
+    f_int = format["int"]
 
     f_r = format["free indices"][0]
 
     # Create temporary values.
-    f1, f2, f3, f4, f5  = [create_symbol(f_tmp(i), CONST) for i in range(0,5)]
+    f1, f2, f3, f4, f5 = [create_symbol(f_tmp(i), CONST) for i in range(0, 5)]
 
     # Get embedded degree.
     embedded_degree = dof_data["embedded_degree"]
 
     # Create helper symbols.
-    symbol_p    = create_symbol(f_r, CONST)
-    symbol_x    = create_symbol(f_x, CONST)
-    symbol_y    = create_symbol(f_y, CONST)
-    symbol_z    = create_symbol(f_z, CONST)
-    int_n1   = f_int(embedded_degree + 1)
+    symbol_p = create_symbol(f_r, CONST)
+    symbol_x = create_symbol(f_x, CONST)
+    symbol_y = create_symbol(f_y, CONST)
+    symbol_z = create_symbol(f_z, CONST)
+    int_n1 = f_int(embedded_degree + 1)
     float_1 = create_float(1)
     float_2 = create_float(2)
-    float_0_5   = create_float(0.5)
-    float_0_25  = create_float(0.25)
+    float_0_5 = create_float(0.5)
+    float_0_25 = create_float(0.25)
 
     # Initialise return code.
     code = [""]
@@ -403,19 +433,19 @@ def _compute_basisvalues(data, dof_data):
     # Get number of members of the expansion set.
     num_mem = dof_data["num_expansion_members"]
     code += [f_comment("Array of basisvalues")]
-    code += [f_decl(f_double, f_component(f_basisvalue, num_mem), f_tensor([0.0]*num_mem))]
+    code += [f_decl(f_double, f_component(f_basisvalue, num_mem), f_tensor([0.0] * num_mem))]
 
     # Declare helper variables, will be removed if not used.
-    code += ["", f_comment("Declare helper variables")] # Keeping this here to avoid changing references
+    code += ["", f_comment("Declare helper variables")]  # Keeping this here to avoid changing references
 
     # Get the element cell name
     element_cellname = data["cellname"]
 
     def _jrc(a, b, n):
-        an = float( ( 2*n+1+a+b)*(2*n+2+a+b))/ float( 2*(n+1)*(n+1+a+b))
-        bn = float( (a*a-b*b) * (2*n+1+a+b))/ float( 2*(n+1)*(2*n+a+b)*(n+1+a+b) )
-        cn = float( (n+a)*(n+b)*(2*n+2+a+b))/ float( (n+1)*(n+1+a+b)*(2*n+a+b) )
-        return (an,bn,cn)
+        an = float((2 * n + 1 + a + b) * (2 * n + 2 + a + b)) / float(2 * (n + 1) * (n + 1 + a + b))
+        bn = float((a * a - b * b) * (2 * n + 1 + a + b)) / float(2 * (n + 1) * (2 * n + a + b) * (n + 1 + a + b))
+        cn = float((n + a) * (n + b) * (2 * n + 2 + a + b)) / float((n + 1) * (n + 1 + a + b) * (2 * n + a + b))
+        return (an, bn, cn)
 
     # 1D
     if (element_cellname == "interval"):
@@ -457,16 +487,16 @@ def _compute_basisvalues(data, dof_data):
                 #        - a4 * result[k-2,:]
 
                 # The below implements the above (with a = b = apb = 0)
-                for r in range(2, embedded_degree+1):
+                for r in range(2, embedded_degree + 1):
 
                     # Define helper variables
-                    a1 = 2.0*r*r*(2.0*r - 2.0)
-                    a3 = ((2.0*r - 2.0)*(2.0*r - 1.0 )*(2.0*r))/a1
-                    a4 = (2.0*(r - 1.0)*(r - 1.0)*(2.0*r))/a1
+                    a1 = 2.0 * r * r * (2.0 * r - 2.0)
+                    a3 = ((2.0 * r - 2.0) * (2.0 * r - 1.0) * (2.0 * r)) / a1
+                    a4 = (2.0 * (r - 1.0) * (r - 1.0) * (2.0 * r)) / a1
 
                     assign_to = f_component(f_basisvalue, r)
-                    assign_from = f_sub([f_mul([f_x, f_component(f_basisvalue, r-1), f_float(a3)]),
-                                         f_mul([f_component(f_basisvalue, r-2), f_float(a4)])])
+                    assign_from = f_sub([f_mul([f_x, f_component(f_basisvalue, r - 1), f_float(a3)]),
+                                         f_mul([f_component(f_basisvalue, r - 2), f_float(a4)])])
                     code += [f_assign(assign_to, assign_from)]
 
         # Scale values.
@@ -480,7 +510,7 @@ def _compute_basisvalues(data, dof_data):
         # Create names.
         basis_k = create_symbol(f_component(f_basisvalue, str(symbol_p)), CONST)
         # Compute value.
-        fac1 = create_symbol( f_sqrt(str(symbol_p + float_0_5)), CONST )
+        fac1 = create_symbol(f_sqrt(str(symbol_p + float_0_5)), CONST)
         lines += [format["imul"](str(basis_k), str(fac1))]
         # Create loop (block of lines).
         code += f_loop(lines, loop_vars)
@@ -493,11 +523,11 @@ def _compute_basisvalues(data, dof_data):
         # f1 = (1.0+2*x+y)/2.0
         # f2 = (1.0 - y) / 2.0
         # f3 = f2**2
-        fac1 = create_fraction(float_1 + float_2*symbol_x + symbol_y, float_2)
+        fac1 = create_fraction(float_1 + float_2 * symbol_x + symbol_y, float_2)
         fac2 = create_fraction(float_1 - symbol_y, float_2)
         code += [f_decl(f_double, str(f1), fac1)]
         code += [f_decl(f_double, str(f2), fac2)]
-        code += [f_decl(f_double, str(f3), f2*f2)]
+        code += [f_decl(f_double, str(f3), f2 * f2)]
 
         code += ["", f_comment("Compute basisvalues")]
         # The initial value basisvalue 0 is always 1.0.
@@ -507,7 +537,7 @@ def _compute_basisvalues(data, dof_data):
         code += [f_assign(f_component(f_basisvalue, 0), f_float(1.0))]
 
         def _idx2d(p, q):
-            return (p + q)*(p + q + 1)//2 + q
+            return (p + q) * (p + q + 1) // 2 + q
 
         # Only continue if the embedded degree is larger than zero.
         if embedded_degree > 0:
@@ -533,8 +563,8 @@ def _compute_basisvalues(data, dof_data):
                     assign_to = f_component(f_basisvalue, rr)
                     ss = _idx2d(r, 0)
                     tt = _idx2d((r - 1), 0)
-                    A = (2*r + 1.0)/(r + 1)
-                    B = r/(1.0 + r)
+                    A = (2 * r + 1.0) / (r + 1)
+                    B = r / (1.0 + r)
                     v1 = f_mul([f_component(f_basisvalue, ss), f_float(A),
                                 str(f1)])
                     v2 = f_mul([f_component(f_basisvalue, tt), f_float(B),
@@ -552,8 +582,8 @@ def _compute_basisvalues(data, dof_data):
                 rr = _idx2d(r, 1)
                 assign_to = f_component(f_basisvalue, rr)
                 ss = _idx2d(r, 0)
-                A = 0.5*(1 + 2*r)
-                B = 0.5*(3 + 2*r)
+                A = 0.5 * (1 + 2 * r)
+                B = 0.5 * (3 + 2 * r)
                 C = f_add([f_float(A), f_mul([f_float(B), str(symbol_y)])])
                 assign_from = f_mul([f_component(f_basisvalue, ss),
                                      f_group(C)])
@@ -574,7 +604,7 @@ def _compute_basisvalues(data, dof_data):
                         rr = _idx2d(r, (s + 1))
                         ss = _idx2d(r, s)
                         tt = _idx2d(r, s - 1)
-                        A, B, C = _jrc(2*r + 1, 0, s)
+                        A, B, C = _jrc(2 * r + 1, 0, s)
                         assign_to = f_component(f_basisvalue, rr)
                         assign_from = f_sub([f_mul([f_component(f_basisvalue, ss), f_group(f_add([f_float(B), f_mul([str(symbol_y), f_float(A)])]))]),
                                              f_mul([f_component(f_basisvalue, tt), f_float(C)])])
@@ -588,7 +618,7 @@ def _compute_basisvalues(data, dof_data):
             for r in range(0, n1):
                 for s in range(0, n1 - r):
                     rr = _idx2d(r, s)
-                    A = (r + 0.5)*(r + s + 1)
+                    A = (r + 0.5) * (r + s + 1)
                     assign_to = f_component(f_basisvalue, rr)
                     code += [f_imul(assign_to, f_sqrt(A))]
 
@@ -599,7 +629,7 @@ def _compute_basisvalues(data, dof_data):
         # def idx(p,q,r):
         #     return (p+q+r)*(p+q+r+1)*(p+q+r+2)//6 + (q+r)*(q+r+1)//2 + r
         def _idx3d(p, q, r):
-            return (p+q+r)*(p+q+r+1)*(p+q+r+2)//6 + (q+r)*(q+r+1)//2 + r
+            return (p + q + r) * (p + q + r + 1) * (p + q + r + 2) // 6 + (q + r) * (q + r + 1) // 2 + r
 
         # FIAT_NEW.expansions.TetrahedronExpansionSet.
 
@@ -610,15 +640,15 @@ def _compute_basisvalues(data, dof_data):
         # factor3 = 0.5 * ( 1 + 2.0 * y + z )
         # factor4 = 0.5 * ( 1 - z )
         # factor5 = factor4 ** 2
-        fac1 = create_product([float_0_5, float_2 + float_2*symbol_x + symbol_y + symbol_z])
+        fac1 = create_product([float_0_5, float_2 + float_2 * symbol_x + symbol_y + symbol_z])
         fac2 = create_product([float_0_25, symbol_y + symbol_z, symbol_y + symbol_z])
-        fac3 = create_product([float_0_5, float_1 + float_2*symbol_y + symbol_z])
+        fac3 = create_product([float_0_5, float_1 + float_2 * symbol_y + symbol_z])
         fac4 = create_product([float_0_5, float_1 - symbol_z])
         code += [f_decl(f_double, str(f1), fac1)]
         code += [f_decl(f_double, str(f2), fac2)]
         code += [f_decl(f_double, str(f3), fac3)]
         code += [f_decl(f_double, str(f4), fac4)]
-        code += [f_decl(f_double, str(f5), f4*f4)]
+        code += [f_decl(f_double, str(f5), f4 * f4)]
 
         code += ["", f_comment("Compute basisvalues")]
         # The initial value basisvalue 0 is always 1.0.
@@ -649,8 +679,8 @@ def _compute_basisvalues(data, dof_data):
                     rr = _idx3d((r + 1), 0, 0)
                     ss = _idx3d(r, 0, 0)
                     tt = _idx3d((r - 1), 0, 0)
-                    A = (2*r + 1.0)/(r + 1)
-                    B = r/(r + 1.0)
+                    A = (2 * r + 1.0) / (r + 1)
+                    B = r / (r + 1.0)
                     assign_to = f_component(f_basisvalue, rr)
                     assign_from = f_sub([f_mul([f_float(A), str(f1), f_component(f_basisvalue, ss)]), f_mul([f_float(B), str(f2), f_component(f_basisvalue, tt)])])
                     code += [f_assign(assign_to, assign_from)]
@@ -689,7 +719,7 @@ def _compute_basisvalues(data, dof_data):
                         rr = _idx3d(r, (s + 1), 0)
                         ss = _idx3d(r, s, 0)
                         tt = _idx3d(r, s - 1, 0)
-                        (A, B, C) = _jrc(2*r + 1, 0, s)
+                        (A, B, C) = _jrc(2 * r + 1, 0, s)
                         assign_to = f_component(f_basisvalue, rr)
                         term0 = f_mul([f_group(f_add([f_mul([f_float(A), str(f3)]), f_mul([f_float(B), str(f4)])])), f_component(f_basisvalue, ss)])
                         term1 = f_mul([f_float(C), str(f5), f_component(f_basisvalue, tt)])
@@ -725,11 +755,11 @@ def _compute_basisvalues(data, dof_data):
                 for r in range(embedded_degree - 1):
                     for s in range(0, embedded_degree - r - 1):
                         for t in range(1, embedded_degree - r - s):
-                            rr = _idx3d(r, s, ( t + 1))
+                            rr = _idx3d(r, s, (t + 1))
                             ss = _idx3d(r, s, t)
                             tt = _idx3d(r, s, t - 1)
 
-                            (A, B, C) = _jrc(2*r + 2*s + 2, 0, t)
+                            (A, B, C) = _jrc(2 * r + 2 * s + 2, 0, t)
                             assign_to = f_component(f_basisvalue, rr)
                             az_b = f_group(f_add([f_float(B), f_mul([f_float(A), str(symbol_z)])]))
                             assign_from = f_sub([f_mul([f_component(f_basisvalue, ss), az_b]), f_mul([f_float(C), f_component(f_basisvalue, tt)])])
@@ -744,13 +774,13 @@ def _compute_basisvalues(data, dof_data):
                 for s in range(embedded_degree - r + 1):
                     for t in range(embedded_degree - r - s + 1):
                         rr = _idx3d(r, s, t)
-                        A = (r + 0.5)*(r + s + 1)*(r + s + t + 1.5)
+                        A = (r + 0.5) * (r + s + 1) * (r + s + t + 1.5)
                         assign_to = f_component(f_basisvalue, rr)
                         multiply_by = f_sqrt(A)
                         myline = f_imul(assign_to, multiply_by)
                         code += [myline]
 
     else:
-        error("Cannot compute basis values for shape: %d" % elemet_cell_domain)
+        error("Cannot compute basis values for shape: %d" % element_cellname)
 
     return code + [""]
