@@ -1,3 +1,26 @@
+# -*- coding: utf-8 -*-
+
+# Copyright (C) 2009-2016 Marie E. Rognes
+#
+# This file is part of FFC.
+#
+# FFC is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# FFC is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
+#
+# Modified by Kristian B. Oelgaard 2010-2011
+# Modified by Anders Logg 2013
+# Modified by Lizao Li 2015, 2016
+
 """Code generation for evaluate_dof.
 
 This module generates the functions evaluate_dof and evaluate_dofs.
@@ -24,29 +47,8 @@ FIAT (functional.pt_dict) in the intermediate representation stage.
 
 """
 
-# Copyright (C) 2009 Marie E. Rognes
-#
-# This file is part of FFC.
-#
-# FFC is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Lesser General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# FFC is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with FFC. If not, see <http://www.gnu.org/licenses/>.
-#
-# Modified by Kristian B. Oelgaard 2010-2011
-# Modified by Anders Logg 2013
-# Modified by Lizao Li 2015
-#
-# First added:  2009-xx-yy
-# Last changed: 2015-03-20
+from collections import OrderedDict
+import six
 
 from ffc.cpp import format, remove_unused
 from ffc.utils import pick_first
@@ -55,35 +57,36 @@ from ufl.permutation import build_component_numbering
 __all__ = ["evaluate_dof_and_dofs", "affine_weights"]
 
 # Prefetch formats:
-comment =   format["comment"]
-declare =   format["declaration"]
-assign =    format["assign"]
+comment = format["comment"]
+declare = format["declaration"]
+assign = format["assign"]
 component = format["component"]
-iadd =      format["iadd"]
-inner =     format["inner product"]
-add =       format["addition"]
-multiply =  format["multiply"]
-J =         format["J"]
-Jinv =      format["inv(J)"]
-detJ =      format["det(J)"](None)
-ret =       format["return"]
-f_i =       format["argument dof num"]
-f_values =  format["argument values"]
-f_double =  format["float declaration"]
-f_vals =    format["dof vals"]
-f_result =  format["dof result"]
-f_y =       format["dof physical coordinates"]
-f_x =       format["vertex_coordinates"]
-f_int =     format["int declaration"]
-f_X =       format["dof X"]
-f_D =       format["dof D"]
-f_W =       format["dof W"]
-f_copy =    format["dof copy"]
-f_r, f_s =  format["free indices"][:2]
-f_loop =    format["generate loop"]
+iadd = format["iadd"]
+inner = format["inner product"]
+add = format["addition"]
+multiply = format["multiply"]
+J = format["J"]
+Jinv = format["inv(J)"]
+detJ = format["det(J)"](None)
+ret = format["return"]
+f_i = format["argument dof num"]
+f_values = format["argument values"]
+f_double = format["float declaration"]
+f_vals = format["dof vals"]
+f_result = format["dof result"]
+f_y = format["dof physical coordinates"]
+f_x = format["coordinate_dofs"]
+f_int = format["int declaration"]
+f_X = format["dof X"]
+f_D = format["dof D"]
+f_W = format["dof W"]
+f_copy = format["dof copy"]
+f_r, f_s = format["free indices"][:2]
+f_loop = format["generate loop"]
 
 map_onto_physical = format["map onto physical"]
 
+
 def evaluate_dof_and_dofs(ir):
     "Generate code for evaluate_dof and evaluate_dof."
 
@@ -94,13 +97,22 @@ def evaluate_dof_and_dofs(ir):
     dof_cases = ["%s\n%s" % (c, ret(r)) for (c, r) in cases]
     dof_code = reqs + format["switch"](f_i, dof_cases, ret(format["float"](0.0)))
 
+    # Construct dict with eval code as keys to remove duplicate eval code
+    cases_opt = OrderedDict((case[0], []) for case in cases)
+    for i, (evl, res) in enumerate(cases):
+        cases_opt[evl].append((i, res))
+
     # Combine each case with assignments for evaluate_dofs
-    dofs_cases = "\n".join("%s\n%s" % (c, format["assign"](component(f_values, i), r))
-                           for (i, (c, r)) in enumerate(cases))
-    dofs_code = reqs + dofs_cases
+    dofs_code = reqs
+    for evl, results in six.iteritems(cases_opt):
+        dofs_code += evl + "\n"
+        for i, res in results:
+            dofs_code += format["assign"](component(f_values, i), res) + "\n"
+    dofs_code = dofs_code.rstrip("\n")
 
     return (dof_code, dofs_code)
 
+
 def _generate_common_code(ir):
 
     # Define necessary geometry information based on the ir
@@ -108,7 +120,7 @@ def _generate_common_code(ir):
 
     # Extract variables
     mappings = ir["mappings"]
-    offsets  = ir["physical_offsets"]
+    offsets = ir["physical_offsets"]
     gdim = ir["geometric_dimension"]
     tdim = ir["topological_dimension"]
 
@@ -118,6 +130,7 @@ def _generate_common_code(ir):
 
     return (reqs, cases)
 
+
 def _required_declarations(ir):
     """Generate code for declaring required variables and geometry
     information.
@@ -141,14 +154,16 @@ def _required_declarations(ir):
     # Check whether Jacobians are necessary.
     needs_inverse_jacobian = any(["contravariant piola" in m
                                   for m in ir["mappings"]])
-    needs_jacobian = any(["covariant piola" in m for m in ir["mappings"]]) or any(["pullback as metric" in m for m in ir["mappings"]])
+    needs_jacobian = any(["covariant piola" in m for m in ir["mappings"]])
 
     # Check if Jacobians are needed
     if not (needs_jacobian or needs_inverse_jacobian):
         return "\n".join(code)
 
-    # Otherwise declare intermediate result variable
-    code.append(declare(f_double, f_result))
+    # Intermediate variable needed for multiple point dofs
+    needs_temporary = any(len(dof) > 1 for dof in ir["dofs"])
+    if needs_temporary:
+        code.append(declare(f_double, f_result))
 
     # Add sufficient Jacobian information. Note: same criterion for
     # needing inverse Jacobian as for needing oriented Jacobian
@@ -161,6 +176,7 @@ def _required_declarations(ir):
 
     return "\n".join(code)
 
+
 def _generate_body(i, dof, mapping, gdim, tdim, offset=0, result=f_result):
     "Generate code for a single dof."
 
@@ -185,7 +201,7 @@ def _generate_body(i, dof, mapping, gdim, tdim, offset=0, result=f_result):
     # Map point onto physical element: y = F_K(x)
     code = []
     for j in range(gdim):
-        y = inner(w, [component(f_x(), (k*gdim + j,)) for k in range(tdim + 1)])
+        y = inner(w, [component(f_x(), (k * gdim + j,)) for k in range(tdim + 1)])
         code.append(assign(component(f_y, j), y))
 
     # Evaluate function at physical point
@@ -204,18 +220,16 @@ def _generate_body(i, dof, mapping, gdim, tdim, offset=0, result=f_result):
 
     # Take inner product between components and weights
     value = add([multiply([w, F[index_map[k]]]) for (w, k) in dof[x]])
-    
-    # Assign value to result variable
-    code.append(assign(result, value))
-    return ("\n".join(code), result)
+
+    # Return eval code and value
+    return ("\n".join(code), value)
 
 
 def _generate_multiple_points_body(i, dof, mapping, gdim, tdim,
                                    offset=0, result=f_result):
-
     "Generate c++ for-loop for multiple points (integral bodies)"
 
-    code = [assign(f_result, 0.0)]
+    code = [assign(result, 0.0)]
     points = list(dof.keys())
     n = len(points)
 
@@ -245,7 +259,7 @@ def _generate_multiple_points_body(i, dof, mapping, gdim, tdim,
     code += [comment("Loop over points")]
 
     # Map the points from the reference onto the physical element
-    #assert(gdim == tdim), \
+    # assert(gdim == tdim), \
     #    "Integral moments not supported for manifolds (yet). Please fix"
     lines_r = [map_onto_physical[tdim][gdim] % {"i": i, "j": f_r}]
 
@@ -265,7 +279,7 @@ def _generate_multiple_points_body(i, dof, mapping, gdim, tdim,
                                 component(f_D(i), (f_r, f_s))),
                       component(f_W(i), (f_r, f_s))])
     # Add value from this point to total result
-    lines_s = [iadd(f_result, value)]
+    lines_s = [iadd(result, value)]
 
     # Generate loop over s and add to r.
     loop_vars_s = [(f_s, 0, len_tokens)]
@@ -278,6 +292,7 @@ def _generate_multiple_points_body(i, dof, mapping, gdim, tdim,
     code = "\n".join(code)
     return code
 
+
 def _change_variables(mapping, gdim, tdim, offset):
     """Generate code for mapping function values according to
     'mapping' and offset.
@@ -309,9 +324,14 @@ def _change_variables(mapping, gdim, tdim, offset):
 
       G(X) = J^T g(x)          i.e  G_i(X) = J^T_ij g(x) = J_ji g_j(x)
 
-    'pullback as metric' mapping for g:
+    'double covariant piola' mapping for g:
 
       G(X) = J^T g(x) J     i.e. G_il(X) = J_ji g_jk(x) J_kl
+
+    'double contravariant piola' mapping for g:
+
+      G(X) = det(J)^2 K g(x) K^T  i.e. G_il(X)=(detJ)^2 K_ij g_jk K_lk
+
     """
 
     # meg: Various mappings must be handled both here and in
@@ -340,8 +360,8 @@ def _change_variables(mapping, gdim, tdim, offset):
             values += [inner(jacobian_column, components)]
         return values
 
-    elif mapping == "pullback as metric":
-        # physical to reference pullback as a metric
+    elif mapping == "double covariant piola":
+        # physical to reference pullback as a covariant 2-tensor
         values = []
         for i in range(tdim):
             for l in range(tdim):
@@ -351,11 +371,22 @@ def _change_variables(mapping, gdim, tdim, offset):
                             for j in range(gdim)]) for k in range(gdim)],
                     [J(k, l, gdim, tdim) for k in range(gdim)])]
         return values
-    
+
+    elif mapping == "double contravariant piola":
+        # physical to reference using double contravariant piola
+        values = []
+        for i in range(tdim):
+            for l in range(tdim):
+                values += [multiply([detJ, detJ, inner(
+                    [inner([Jinv(i, j, gdim, tdim) for j in range(gdim)],
+                           [component(f_vals, j * tdim + k + offset)
+                            for j in range(gdim)]) for k in range(gdim)],
+                    [Jinv(l, k, gdim, tdim) for k in range(gdim)])])]
+        return values
+
     else:
         raise Exception("The mapping (%s) is not allowed" % mapping)
 
-    return code
 
 def affine_weights(dim):
     "Compute coefficents for mapping from reference to physical element"
