@@ -1,4 +1,6 @@
-# Copyright (C) 2009-2013 Kristian B. Oelgaard and Anders Logg
+# -*- coding: utf-8 -*-
+
+# Copyright (C) 2009-2016 Kristian B. Oelgaard and Anders Logg
 #
 # This file is part of FFC.
 #
@@ -17,24 +19,23 @@
 #
 # Modified by Garth N. Wells, 2009.
 # Modified by Marie Rognes, 2009-2013.
-# Modified by Martin Alnaes, 2013
-# Modified by Lizao Li, 2015
+# Modified by Martin Sandve Alnæs, 2013
+# Modified by Lizao Li, 2015, 2016
 
 # Python modules
-from numpy import array
 import six
+import numpy
+from numpy import array
 
 # UFL and FIAT modules
 import ufl
 from ufl.utils.sorting import sorted_by_key
-import FIAT
 
-from FIAT.trace import DiscontinuousLagrangeTrace
+import FIAT
+from FIAT.hdiv_trace import HDivTrace
 
 # FFC modules
-from ffc.log import debug, error, ffc_assert
-from ffc.quadratureelement import QuadratureElement as FFCQuadratureElement
-
+from ffc.log import debug, error
 
 from ffc.mixedelement import MixedElement
 from ffc.restrictedelement import RestrictedElement
@@ -49,7 +50,7 @@ supported_families = ("Brezzi-Douglas-Marini",
                       "Crouzeix-Raviart",
                       "Discontinuous Lagrange",
                       "Discontinuous Raviart-Thomas",
-                      "Discontinuous Lagrange Trace",
+                      "HDiv Trace",
                       "Lagrange",
                       "Lobatto",
                       "Nedelec 1st kind H(curl)",
@@ -59,22 +60,26 @@ supported_families = ("Brezzi-Douglas-Marini",
                       "Real",
                       "Bubble",
                       "Quadrature",
-                      "Regge")
+                      "Regge",
+                      "Hellan-Herrmann-Johnson")
 
 # Cache for computed elements
 _cache = {}
 
+
 def reference_cell(dim):
     if isinstance(dim, int):
         return FIAT.ufc_simplex(dim)
     else:
         return FIAT.ufc_simplex(cellname2dim[dim])
 
+
 def reference_cell_vertices(dim):
     "Return dict of coordinates of reference cell vertices for this 'dim'."
     cell = reference_cell(dim)
     return cell.get_vertices()
 
+
 def create_element(ufl_element):
 
     # Create element signature for caching (just use UFL element)
@@ -111,38 +116,30 @@ def create_element(ufl_element):
 
     return element
 
+
 def _create_fiat_element(ufl_element):
     "Create FIAT element corresponding to given finite element."
 
     # Get element data
     family = ufl_element.family()
-    domain, = ufl_element.domains() # Assuming single domain
-    cellname = domain.cell().cellname() # Assuming single cell in domain
+    cell = ufl_element.cell()
+    cellname = cell.cellname()
     degree = ufl_element.degree()
 
     # Check that FFC supports this element
-    ffc_assert(family in supported_families,
-               "This element family (%s) is not supported by FFC." % family)
+    if family not in supported_families:
+        error("This element family (%s) is not supported by FFC." % family)
 
     # Handle the space of the constant
     if family == "Real":
-        dg0_element = ufl.FiniteElement("DG", domain, 0)
+        dg0_element = ufl.FiniteElement("DG", cell, 0)
         constant = _create_fiat_element(dg0_element)
         element = SpaceOfReals(constant)
 
-    # Handle the specialized time elements
-    elif family == "Lobatto" :
-        from ffc.timeelements import LobattoElement as FFCLobattoElement
-        element = FFCLobattoElement(ufl_element.degree())
-
-    elif family == "Radau" :
-        from ffc.timeelements import RadauElement as FFCRadauElement
-        element = FFCRadauElement(ufl_element.degree())
-
     # FIXME: AL: Should this really be here?
     # Handle QuadratureElement
     elif family == "Quadrature":
-        element = FFCQuadratureElement(ufl_element)
+        element = QuadratureElement(ufl_element)
 
     else:
         # Create FIAT cell
@@ -151,11 +148,11 @@ def _create_fiat_element(ufl_element):
         # Handle Bubble element as RestrictedElement of P_{k} to interior
         if family == "Bubble":
             V = FIAT.supported_elements["Lagrange"](fiat_cell, degree)
-            dim = domain.topological_dimension()
-            return RestrictedElement(V, _indices(V, "interior", dim), None)
+            tdim = cell.topological_dimension()
+            return RestrictedElement(V, _indices(V, "interior", tdim), None)
 
         # Check if finite element family is supported by FIAT
-        if not family in FIAT.supported_elements:
+        if family not in FIAT.supported_elements:
             error("Sorry, finite element of type \"%s\" are not supported by FIAT.", family)
 
         # Create FIAT finite element
@@ -165,29 +162,61 @@ def _create_fiat_element(ufl_element):
         else:
             element = ElementClass(fiat_cell, degree)
 
-    # Consistency check between UFL and FIAT elements. This will not hold for elements
-    # where the reference value shape is different from the global value shape, i.e.
-    # RT elements on a triangle in 3D.
-    #ffc_assert(element.value_shape() == ufl_element.value_shape(),
-    #           "Something went wrong in the construction of FIAT element from UFL element." + \
-    #           "Shapes are %s and %s." % (element.value_shape(), ufl_element.value_shape()))
+    # Consistency check between UFL and FIAT elements.
+    if element.value_shape() != ufl_element.reference_value_shape():
+        error("Something went wrong in the construction of FIAT element from UFL element." +
+              "Shapes are %s and %s." % (element.value_shape(), ufl_element.reference_value_shape()))
 
     return element
 
-def create_quadrature(shape, num_points):
-    """
-    Generate quadrature rule (points, weights) for given shape with
-    num_points points in each direction.
-    """
 
+def create_quadrature(shape, degree, scheme="default"):
+    """
+    Generate quadrature rule (points, weights) for given shape
+    that will integrate an polynomial of order 'degree' exactly.
+    """
     if isinstance(shape, int) and shape == 0:
-        return ([()], array([1.0,]))
+        return (numpy.zeros((1, 0)), numpy.ones((1,)))
 
     if shape in cellname2dim and cellname2dim[shape] == 0:
-        return ([()], array([1.0,]))
+        return (numpy.zeros((1, 0)), numpy.ones((1,)))
+
+    if scheme == "vertex":
+        # The vertex scheme, i.e., averaging the function value in the vertices
+        # and multiplying with the simplex volume, is only of order 1 and
+        # inferior to other generic schemes in terms of error reduction.
+        # Equation systems generated with the vertex scheme have some
+        # properties that other schemes lack, e.g., the mass matrix is
+        # a simple diagonal matrix. This may be prescribed in certain cases.
+        if degree > 1:
+            from warnings import warn
+            warn(("Explicitly selected vertex quadrature (degree 1), "
+                 + "but requested degree is %d.") % degree)
+        if shape == "tetrahedron":
+            return (array([[0.0, 0.0, 0.0],
+                           [1.0, 0.0, 0.0],
+                           [0.0, 1.0, 0.0],
+                           [0.0, 0.0, 1.0]]),
+                    array([1.0 / 24.0, 1.0 / 24.0, 1.0 / 24.0, 1.0 / 24.0])
+                    )
+        elif shape == "triangle":
+            return (array([[0.0, 0.0],
+                           [1.0, 0.0],
+                           [0.0, 1.0]]),
+                    array([1.0 / 6.0, 1.0 / 6.0, 1.0 / 6.0])
+                    )
+        elif shape == "interval":
+            # Trapezoidal rule.
+            return (array([[0.0],
+                           [1.0]]),
+                    array([1.0 / 2.0, 1.0 / 2.0])
+                    )
+
+    quad_rule = FIAT.create_quadrature(reference_cell(shape), degree, scheme)
+    points = numpy.asarray(quad_rule.get_points())
+    weights = numpy.asarray(quad_rule.get_weights())
+    return points, weights
 
-    quad_rule = FIAT.make_quadrature(reference_cell(shape), num_points)
-    return quad_rule.get_points(), quad_rule.get_weights()
 
 def map_facet_points(points, facet):
     """
@@ -210,64 +239,67 @@ def map_facet_points(points, facet):
 
     # Extract vertex coordinates from cell and map of facet index to
     # indicent vertex indices
-    vertex_coordinates = fiat_cell.get_vertices()
-    facet_vertices = fiat_cell.get_topology()[dim-1]
+    coordinate_dofs = fiat_cell.get_vertices()
+    facet_vertices = fiat_cell.get_topology()[dim - 1]
 
-    #vertex_coordinates = \
+    # coordinate_dofs = \
     #    {1: ((0.,), (1.,)),
     #     2: ((0., 0.), (1., 0.), (0., 1.)),
     #     3: ((0., 0., 0.), (1., 0., 0.),(0., 1., 0.), (0., 0., 1))}
 
     # Facet vertices
-    #facet_vertices = \
+    # facet_vertices = \
     #    {2: ((1, 2), (0, 2), (0, 1)),
     #     3: ((1, 2, 3), (0, 2, 3), (0, 1, 3), (0, 1, 2))}
 
     # Compute coordinates and map the points
-    coordinates = [vertex_coordinates[v] for v in facet_vertices[facet]]
+    coordinates = [coordinate_dofs[v] for v in facet_vertices[facet]]
     new_points = []
     for point in points:
         w = (1.0 - sum(point),) + tuple(point)
-        x = tuple(sum([w[i]*array(coordinates[i]) for i in range(len(w))]))
+        x = tuple(sum([w[i] * array(coordinates[i]) for i in range(len(w))]))
         new_points += [x]
 
     return new_points
 
-def _extract_elements(ufl_element, domain=None):
+
+def _extract_elements(ufl_element, restriction_domain=None):
     "Recursively extract un-nested list of (component) elements."
 
     elements = []
     if isinstance(ufl_element, ufl.MixedElement):
         for sub_element in ufl_element.sub_elements():
-            elements += _extract_elements(sub_element, domain)
+            elements += _extract_elements(sub_element, restriction_domain)
         return elements
 
     # Handle restricted elements since they might be mixed elements too.
     if isinstance(ufl_element, ufl.RestrictedElement):
-        base_element = ufl_element.element()
-        restriction = ufl_element.cell_restriction()
-        return _extract_elements(base_element, restriction)
+        base_element = ufl_element.sub_element()
+        restriction_domain = ufl_element.restriction_domain()
+        return _extract_elements(base_element, restriction_domain)
 
-    if domain:
-        ufl_element = ufl.RestrictedElement(ufl_element, domain)
+    if restriction_domain:
+        ufl_element = ufl.RestrictedElement(ufl_element, restriction_domain)
 
     elements += [create_element(ufl_element)]
 
     return elements
 
+
 def _create_restricted_element(ufl_element):
     "Create an FFC representation for an UFL RestrictedElement."
 
     if not isinstance(ufl_element, ufl.RestrictedElement):
         error("create_restricted_element expects an ufl.RestrictedElement")
 
-    base_element = ufl_element.element()
-    restriction_domain = ufl_element.cell_restriction()
+    base_element = ufl_element.sub_element()
+    restriction_domain = ufl_element.restriction_domain()
 
     # If simple element -> create RestrictedElement from fiat_element
     if isinstance(base_element, ufl.FiniteElement):
         element = _create_fiat_element(base_element)
-        return RestrictedElement(element, _indices(element, restriction_domain), restriction_domain)
+        tdim = ufl_element.cell().topological_dimension()
+        return RestrictedElement(element, _indices(element, restriction_domain, tdim), restriction_domain)
 
     # If restricted mixed element -> convert to mixed restricted element
     if isinstance(base_element, ufl.MixedElement):
@@ -276,7 +308,8 @@ def _create_restricted_element(ufl_element):
 
     error("Cannot create restricted element from %s" % str(ufl_element))
 
-def _indices(element, restriction_domain, dim=0):
+
+def _indices(element, restriction_domain, tdim):
     "Extract basis functions indices that correspond to restriction_domain."
 
     # FIXME: The restriction_domain argument in FFC/UFL needs to be re-thought and
@@ -284,31 +317,30 @@ def _indices(element, restriction_domain, dim=0):
 
     # If restriction_domain is "interior", pick basis functions associated with
     # cell.
-    if restriction_domain == "interior" and dim:
-        return element.entity_dofs()[dim][0]
+    if restriction_domain == "interior":
+        return element.entity_dofs()[tdim][0]
 
-    # If restriction_domain is a ufl.Cell, pick basis functions associated with
+    # Pick basis functions associated with
     # the topological degree of the restriction_domain and of all lower
     # dimensions.
-    if isinstance(restriction_domain, ufl.Cell):
-        dim = restriction_domain.topological_dimension()
-        entity_dofs = element.entity_dofs()
-        indices = []
-        for dim in range(restriction_domain.topological_dimension() + 1):
-            entities = entity_dofs[dim]
-            for (entity, index) in sorted_by_key(entities):
-                indices += index
-        return indices
-
-    # Just extract all indices to make handling in RestrictedElement
-    # uniform.
-    #elif isinstance(restriction_domain, ufl.Measure):
-    #    indices = []
-    #    entity_dofs = element.entity_dofs()
-    #    for dim, entities in entity_dofs.items():
-    #        for entity, index in entities.items():
-    #            indices += index
-    #    return indices
-
+    if restriction_domain == "facet":
+        rdim = tdim - 1
+    elif restriction_domain == "face":
+        rdim = 2
+    elif restriction_domain == "edge":
+        rdim = 1
+    elif restriction_domain == "vertex":
+        rdim = 0
     else:
         error("Restriction to domain: %s, is not supported." % repr(restriction_domain))
+
+    entity_dofs = element.entity_dofs()
+    indices = []
+    for dim in range(rdim + 1):
+        entities = entity_dofs[dim]
+        for (entity, index) in sorted_by_key(entities):
+            indices += index
+    return indices
+
+# Import FFC module with circular dependency
+from ffc.quadratureelement import QuadratureElement
