@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 "Code generation for interpolate_vertex_values."
 
 # Copyright (C) 2009 Marie E. Rognes
@@ -18,31 +19,33 @@
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Kristian B. Oelgaard 2010
-# Modified by Lizao Li 2015
+# Modified by Lizao Li 2015, 2016
 #
-# Last changed: 2015-03-25
+# Last changed: 2016-08-17
 
+from six import string_types
 from ffc.cpp import format, remove_unused
 
 # Extract code manipulation formats
-inner =     format["inner product"]
+inner = format["inner product"]
 component = format["component"]
-assign =    format["assign"]
-multiply =  format["multiply"]
+assign = format["assign"]
+multiply = format["multiply"]
 
 # Extract formats for the Jacobians
-J =       format["J"]
-Jinv =    format["inv(J)"]
+J = format["J"]
+Jinv = format["inv(J)"]
 invdetJ = format["inverse"](format["det(J)"](None))
 
-f_dof_values =    format["argument dof values"]
+f_dof_values = format["argument dof values"]
 f_vertex_values = format["argument vertex values"]
 
+
 def interpolate_vertex_values(ir):
     "Generate code for interpolate_vertex_values."
 
     # Handle unsupported elements.
-    if isinstance(ir, str):
+    if isinstance(ir, string_types):
         return format["exception"]("interpolate_vertex_values: %s" % ir)
 
     # Add code for Jacobian if necessary
@@ -105,7 +108,8 @@ def _interpolate_vertex_values_element(data, gdim, tdim, total_value_size,
         # Create code for each vertex x_j
         for (j, values_at_vertex) in enumerate(vertex_values):
 
-            if value_size == 1: values_at_vertex = [values_at_vertex]
+            if value_size == 1:
+                values_at_vertex = [values_at_vertex]
 
             # Map basis functions using appropriate mapping
             components = change_of_variables(values_at_vertex, k)
@@ -117,7 +121,7 @@ def _interpolate_vertex_values_element(data, gdim, tdim, total_value_size,
             value = inner(dof_values, components)
 
             # Assign value to correct vertex
-            index = j*total_value_size + (k + value_offset)
+            index = j * total_value_size + (k + value_offset)
             code.append(assign(component(f_vertex_values, index), value))
 
     return "\n".join(code)
@@ -155,10 +159,14 @@ def _change_variables(mapping, gdim, tdim, space_dim):
 
       g(x) = K^T G(X)              i.e   g_i(x) = K^T_ij G_j(X) = K_ji G_j(X)
 
-    'pullback as metric' mapping for f:
-    
+    'double covariant piola' mapping for f:
+
       g_il(x) = K_{ji} G_{jk} K_{kl}
 
+    'double contravariant piola' mapping for g:
+
+      g_il(x) = (det(J))^(-2) J_ij G_jk(X) J_lk
+
     """
 
     if mapping is "affine":
@@ -171,13 +179,21 @@ def _change_variables(mapping, gdim, tdim, space_dim):
         change_of_variables = lambda G, i: [inner([Jinv(j, i, tdim, gdim) for j in range(tdim)],
                                                   [G[j][index] for j in range(tdim)])
                                             for index in range(space_dim)]
-    elif mapping == "pullback as metric":
+    elif mapping == "double covariant piola":
         change_of_variables = lambda G, i: [
             inner([inner([Jinv(j, i // tdim, tdim, gdim) for j in range(tdim)],
                          [G[j][k][index] for j in range(tdim)])
                    for k in range(tdim)],
                   [Jinv(k, i % tdim, tdim, gdim) for k in range(tdim)])
             for index in range(space_dim)]
+    elif mapping == "double contravariant piola":
+        change_of_variables = lambda G, i: [
+            multiply([invdetJ, invdetJ, inner(
+                [inner([J(i // tdim, j, tdim, gdim) for j in range(tdim)],
+                         [G[j][k][index] for j in range(tdim)])
+                   for k in range(tdim)],
+                  [J(i % tdim, k, tdim, gdim) for k in range(tdim)])])
+            for index in range(space_dim)]
     else:
         raise Exception("No such mapping: %s accepted" % mapping)
     return change_of_variables
