@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """This module provides functions used by the FFC implementation to
 output messages. These may be redirected by the user of FFC.
 
@@ -27,9 +28,9 @@ is a wrapper for the standard Python logging module.
 # UFL modules
 from ufl.log import Logger
 from ufl.log import log_functions
-from ufl.log import INFO, DEBUG, ERROR, CRITICAL
-from ufl.common import dstr, tstr
+from ufl.log import INFO, DEBUG, WARNING, ERROR, CRITICAL
 from ufl.utils.sorting import sorted_by_key
+from ufl.utils.formatting import dstr, tstr
 
 # Create FFC logger
 ffc_logger = Logger("FFC")
@@ -39,6 +40,8 @@ for foo in log_functions:
     exec("%s = lambda *message : ffc_logger.%s(*message)" % (foo, foo))
 
 # Assertion, copied from UFL
+
+
 def ffc_assert(condition, *message):
     "Assert that condition is true and otherwise issue an error with given message."
     condition or error(*message)
@@ -48,27 +51,33 @@ set_level(INFO)
 
 #--- Specialized FFC debugging tools ---
 
+
 def debug_dict(d, title=""):
     "Pretty-print dictionary."
-    if not title: title = "Dictionary"
+    if not title:
+        title = "Dictionary"
     info("")
     begin(title)
     info("")
     for (key, value) in sorted_by_key(d):
         info(key)
-        info("-"*len(key))
+        info("-" * len(key))
         info(str(value))
         info("")
     end()
 
+
 def debug_ir(ir, name=""):
     "Debug intermediate representation."
     title = "Intermediate representation"
-    if name: title += " (%s)" % str(name)
+    if name:
+        title += " (%s)" % str(name)
     debug_dict(ir, title)
 
+
 def debug_code(code, name=""):
     "Debug generated code."
     title = "Generated code"
-    if name: title += " (%s)" % str(name)
+    if name:
+        title += " (%s)" % str(name)
     debug_dict(code, title)
