@@ -1,4 +1,6 @@
-# Copyright (C) 2005-2010 Anders Logg
+# -*- coding: utf-8 -*-
+
+# Copyright (C) 2005-2016 Anders Logg
 #
 # This file is part of FFC.
 #
@@ -19,8 +21,6 @@
 # Modified by Marie E. Rognes, 2007-2010
 # Modified by Kristian B. Oelgaard, 2010
 # Modified by Lizao Li, 2015
-#
-# Last changed: 2015-04-25
 
 # Python modules
 import numpy
@@ -31,7 +31,9 @@ from ffc.log import error
 # UFL utils
 from ufl.utils.sequences import product
 
+
 class MixedElement:
+
     "Create a FFC mixed element from a list of FFC/FIAT elements."
 
     def __init__(self, elements):
@@ -46,7 +48,7 @@ class MixedElement:
 
     def value_shape(self):
         # Values of Tensor elements are flattened in MixedElements
-        num_comps = lambda x: numpy.prod(x) if x else 1 
+        num_comps = lambda x: numpy.prod(x) if x else 1
         return (sum(num_comps(e.value_shape()) or 1 for e in self._elements),)
 
     def entity_dofs(self):
@@ -102,7 +104,7 @@ class MixedElement:
             for dtuple in table.keys():
 
                 # Insert zeros if necessary (should only happen first time)
-                if not dtuple in mixed_table:
+                if dtuple not in mixed_table:
                     # NOTE: It is super important to create a new numpy.zeros
                     # instance to avoid manipulating a numpy reference in case
                     # it is created outside the loop.
@@ -118,6 +120,7 @@ class MixedElement:
 
 #--- Utility functions ---
 
+
 def _combine_entity_dofs(elements):
     """
     Combine the entity_dofs from a list of elements into a combined
@@ -152,6 +155,7 @@ def _combine_entity_dofs(elements):
         offset += e.space_dimension()
     return entity_dofs
 
+
 def _num_components(element):
     "Compute number of components for element."
     return product(element.value_shape())
