@@ -1,12 +1,6 @@
-"""
-Compiler stage 5: optimization
-------------------------------
+# -*- coding: utf-8 -*-
 
-This module implements the optimization of an intermediate code
-representation.
-"""
-
-# Copyright (C) 2009-2013 Anders Logg
+# Copyright (C) 2009-2016 Anders Logg
 #
 # This file is part of FFC.
 #
@@ -23,15 +17,21 @@ representation.
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
-# Modified by Martin Alnaes, 2013
-#
-# First added:  2009-12-22
-# Last changed: 2013-02-10
+# Modified by Martin Sandve Alnæs, 2013
+
+"""
+Compiler stage 5: optimization
+------------------------------
+
+This module implements the optimization of an intermediate code
+representation.
+"""
 
 # FFC modules
 from ffc.log import info, begin, end
 from ffc.representation import pick_representation
 
+
 def optimize_ir(ir, parameters):
     "Optimize intermediate form representation."
 
@@ -44,14 +44,15 @@ def optimize_ir(ir, parameters):
         return ir
 
     # Extract representations
-    ir_elements, ir_dofmaps, ir_integrals, ir_forms = ir
+    ir_elements, ir_dofmaps, ir_coordinate_mappings, ir_integrals, ir_forms = ir
 
     # Iterate over integrals
     oir_integrals = [_optimize_integral_ir(ir, parameters) for ir in ir_integrals]
 
     end()
 
-    return ir_elements, ir_dofmaps, oir_integrals, ir_forms
+    return ir_elements, ir_dofmaps, ir_coordinate_mappings, oir_integrals, ir_forms
+
 
 def _optimize_integral_ir(ir, parameters):
     "Compute optimized intermediate represention of integral."
