@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 "This file implements a class to represent a product."
 
 # Copyright (C) 2009-2010 Kristian B. Oelgaard
@@ -20,21 +21,26 @@
 # First added:  2009-07-12
 # Last changed: 2010-03-11
 
-# FFC modules.
+from functools import reduce
+
+# FFC modules
 from ffc.log import error
 from ffc.cpp import format
 
-# FFC quadrature modules.
+# FFC quadrature modules
 from .symbolics import create_float
 from .symbolics import create_product
 from .symbolics import create_sum
 from .symbolics import create_fraction
 from .expr import Expr
-from functools import reduce
 
-#class Product(object):
+# FFC quadrature modules
+from .floatvalue import FloatValue
+
+
 class Product(Expr):
     __slots__ = ("vrs", "_expanded")
+
     def __init__(self, variables):
         """Initialise a Product object, it derives from Expr and contains
         the additional variables:
@@ -66,55 +72,57 @@ class Product(Expr):
                     break
 
                 # Collect floats into one variable
-                if var._prec == 0: # float
+                if var._prec == 0:  # float
                     float_val *= var.val
                     continue
-                # Take care of product such that we don't create nested products.
-                elif var._prec == 2: # prod
-#                    if var.vrs[0]._prec == 0:
-#                        float_val *= var.vrs[0].val
-#                        self.vrs += var.vrs[1:]
-#                        continue
-#                    self.vrs += var.vrs
-#                    continue
+                # Take care of product such that we don't create
+                # nested products.
+                elif var._prec == 2:  # prod
                     # If expanded product is a float, just add it.
                     if var._expanded and var._expanded._prec == 0:
                         float_val *= var._expanded.val
-                    # If expanded product is symbol, this product is still expanded and add symbol.
+                    # If expanded product is symbol, this product is
+                    # still expanded and add symbol.
                     elif var._expanded and var._expanded._prec == 1:
                         self.vrs.append(var._expanded)
-                    # If expanded product is still a product, add the variables.
+                    # If expanded product is still a product, add the
+                    # variables.
                     elif var._expanded and var._expanded._prec == 2:
-#                        self.vrs.append(var)
-                        # Add copies of the variables of other product (collect floats).
+                        # Add copies of the variables of other product
+                        # (collect floats).
                         if var._expanded.vrs[0]._prec == 0:
                             float_val *= var._expanded.vrs[0].val
                             self.vrs += var._expanded.vrs[1:]
                             continue
                         self.vrs += var._expanded.vrs
-                    # If expanded product is a sum or fraction, we must expand this product later.
+                    # If expanded product is a sum or fraction, we
+                    # must expand this product later.
                     elif var._expanded and var._expanded._prec in (3, 4):
                         self._expanded = False
                         self.vrs.append(var._expanded)
-                    # Else the product is not expanded, and we must expand this one later
+                    # Else the product is not expanded, and we must
+                    # expand this one later
                     else:
                         self._expanded = False
-                        # Add copies of the variables of other product (collect floats).
+                        # Add copies of the variables of other product
+                        # (collect floats).
                         if var.vrs[0]._prec == 0:
                             float_val *= var.vrs[0].val
                             self.vrs += var.vrs[1:]
                             continue
                         self.vrs += var.vrs
                     continue
-                # If we have sums or fractions in the variables the product is not expanded.
-                elif var._prec in (3, 4): # sum or frac
+                # If we have sums or fractions in the variables the
+                # product is not expanded.
+                elif var._prec in (3, 4):  # sum or frac
                     self._expanded = False
 
-                # Just add any variable at this point to list of new vars.
+                # Just add any variable at this point to list of new
+                # vars.
                 self.vrs.append(var)
 
-            # If value is 1 there is no need to include it, unless it is the
-            # only parameter left i.e., 2*0.5 = 1.
+            # If value is 1 there is no need to include it, unless it
+            # is the only parameter left i.e., 2*0.5 = 1.
             if float_val and float_val != 1.0:
                 self.val = float_val
                 self.vrs.append(create_float(float_val))
@@ -138,25 +146,27 @@ class Product(Expr):
         # Sort the variables such that comparisons work.
         self.vrs.sort()
 
-        # Compute the representation now, such that we can use it directly
-        # in the __eq__ and __ne__ methods (improves performance a bit, but
-        # only when objects are cached).
+        # Compute the representation now, such that we can use it
+        # directly in the __eq__ and __ne__ methods (improves
+        # performance a bit, but only when objects are cached).
         self._repr = "Product([%s])" % ", ".join([v._repr for v in self.vrs])
 
         # Use repr as hash value.
         self._hash = hash(self._repr)
 
-        # Store self as expanded value, if we did not encounter any sums or fractions.
+        # Store self as expanded value, if we did not encounter any
+        # sums or fractions.
         if self._expanded:
             self._expanded = self
 
     # Print functions.
     def __str__(self):
         "Simple string representation which will appear in the generated code."
-        # If we have more than one variable and the first float is -1 exlude the 1.
+        # If we have more than one variable and the first float is -1
+        # exlude the 1.
         if len(self.vrs) > 1 and self.vrs[0]._prec == 0 and self.vrs[0].val == -1.0:
             # Join string representation of members by multiplication
-            return   format["sub"](["", format["mul"]([str(v) for v in self.vrs[1:]])])
+            return format["sub"](["", format["mul"]([str(v) for v in self.vrs[1:]])])
         return format["mul"]([str(v) for v in self.vrs])
 
     # Binary operators.
@@ -167,24 +177,28 @@ class Product(Expr):
         if other._prec == 2 and self.get_vrs() == other.get_vrs():
             # Return expanded product, to get rid of 3*x + -2*x -> x, not 1*x.
             return create_product([create_float(self.val + other.val)] + list(self.get_vrs())).expand()
-        # if self == 2*x and other == x return 3*x.
-        elif other._prec == 1: # sym
+
+        elif other._prec == 1:  # sym
             if self.get_vrs() == (other,):
-                # Return expanded product, to get rid of -x + x -> 0, not product(0).
-                return create_product([create_float(self.val + 1.0), other]).expand()
+                # Return expanded product, to get rid of -x + x -> 0,
+                # not product(0).
+                return create_product([create_float(self.val + 1.0),
+                                       other]).expand()
         # Return sum
         return create_sum([self, other])
 
     def __sub__(self, other):
         "Subtract other objects."
         if other._prec == 2 and self.get_vrs() == other.get_vrs():
-            # Return expanded product, to get rid of 3*x + -2*x -> x, not 1*x.
+            # Return expanded product, to get rid of 3*x + -2*x -> x,
+            # not 1*x.
             return create_product([create_float(self.val - other.val)] + list(self.get_vrs())).expand()
-        # if self == 2*x and other == x return 3*x.
-        elif other._prec == 1: # sym
+        elif other._prec == 1:  # sym
             if self.get_vrs() == (other,):
-                # Return expanded product, to get rid of -x + x -> 0, not product(0).
-                return create_product([create_float(self.val - 1.0), other]).expand()
+                # Return expanded product, to get rid of -x + x -> 0,
+                # not product(0).
+                return create_product([create_float(self.val - 1.0),
+                                       other]).expand()
         # Return sum
         return create_sum([self, create_product([FloatValue(-1), other])])
 
@@ -195,12 +209,12 @@ class Product(Expr):
             return create_float(0)
 
         # If other is a Sum or Fraction let them handle it.
-        if other._prec in (3, 4): # sum or frac
+        if other._prec in (3, 4):  # sum or frac
             return other.__mul__(self)
 
-        # NOTE: We expect expanded sub-expressions with no nested operators.
-        # Create new product adding float or symbol.
-        if other._prec in (0, 1): # float or sym
+        # NOTE: We expect expanded sub-expressions with no nested
+        # operators.  Create new product adding float or symbol.
+        if other._prec in (0, 1):  # float or sym
             return create_product(self.vrs + [other])
         # Create new product adding all variables from other Product.
         return create_product(self.vrs + other.vrs)
@@ -216,10 +230,11 @@ class Product(Expr):
             return self.vrs[0]
 
         # If other is a Sum we can only return a fraction.
-        # NOTE: Expect that other is expanded i.e., x + x -> 2*x which can be handled
+        # NOTE: Expect that other is expanded i.e., x + x -> 2*x which
+        # can be handled
         # TODO: Fix x / (x + x*y) -> 1 / (1 + y).
         # Or should this be handled when reducing a fraction?
-        if other._prec == 3: # sum
+        if other._prec == 3:  # sum
             return create_fraction(self, other)
 
         # Handle division by FloatValue, Symbol, Product and Fraction.
@@ -229,20 +244,21 @@ class Product(Expr):
         num = self.vrs[:]
         denom = []
         # Add floatvalue, symbol and products to the list of denominators.
-        if other._prec in (0, 1): # float or sym
+        if other._prec in (0, 1):  # float or sym
             denom = [other]
-        elif other._prec == 2: # prod
+        elif other._prec == 2:  # prod
             # Get copy.
             denom = other.vrs[:]
         # fraction.
         else:
             error("Did not expected to divide by fraction.")
 
-        # Loop entries in denominator and remove from numerator (and denominator).
+        # Loop entries in denominator and remove from numerator (and
+        # denominator).
         for d in denom[:]:
             # Add the inverse of a float to the numerator and continue.
-            if d._prec == 0: # float
-                num.append(create_float(1.0/d.val))
+            if d._prec == 0:  # float
+                num.append(create_float(1.0 / d.val))
                 denom.remove(d)
                 continue
             if d in num:
@@ -257,7 +273,8 @@ class Product(Expr):
             num = create_product(num).expand()
         elif num:
             num = num[0]
-        # If all variables in the numerator has been eliminated we need to add '1'.
+        # If all variables in the numerator has been eliminated we
+        # need to add '1'.
         else:
             num = create_float(1)
 
@@ -265,7 +282,8 @@ class Product(Expr):
             return create_fraction(num, create_product(denom))
         elif denom:
             return create_fraction(num, denom[0])
-        # If we no longer have a denominater, just return the numerator.
+        # If we no longer have a denominater, just return the
+        # numerator.
         return num
 
     __div__ = __truediv__
@@ -274,8 +292,8 @@ class Product(Expr):
     def expand(self):
         "Expand all members of the product."
         # If we just have one variable, compute the expansion of it
-        # (it is not a Product, so it should be safe). We need this to get
-        # rid of Product([Symbol]) type expressions.
+        # (it is not a Product, so it should be safe). We need this to
+        # get rid of Product([Symbol]) type expressions.
         if len(self.vrs) == 1:
             self._expanded = self.vrs[0].expand()
             return self._expanded
@@ -284,34 +302,35 @@ class Product(Expr):
         if self._expanded:
             return self._expanded
 
-        # Sort variables such that we don't call the '*' operator more than we have to.
+        # Sort variables such that we don't call the '*' operator more
+        # than we have to.
         float_syms = []
         sum_fracs = []
         for v in self.vrs:
-            if v._prec in (0, 1): # float or sym
+            if v._prec in (0, 1):  # float or sym
                 float_syms.append(v)
                 continue
             exp = v.expand()
 
             # If the expanded expression is a float, sym or product,
             # we can add the variables.
-            if exp._prec in (0, 1): # float or sym
+            if exp._prec in (0, 1):  # float or sym
                 float_syms.append(exp)
-            elif exp._prec == 2: # prod
+            elif exp._prec == 2:  # prod
                 float_syms += exp.vrs
             else:
                 sum_fracs.append(exp)
         # If we have floats or symbols add the symbols to the rest as a single
         # product (for speed).
         if len(float_syms) > 1:
-            sum_fracs.append( create_product(float_syms) )
+            sum_fracs.append(create_product(float_syms))
         elif float_syms:
             sum_fracs.append(float_syms[0])
 
         # Use __mult__ to reduce list to one single variable.
-        # TODO: Can this be done more efficiently without creating all the
-        # intermediate variables?
-        self._expanded = reduce(lambda x,y: x*y, sum_fracs)
+        # TODO: Can this be done more efficiently without creating all
+        # the intermediate variables?
+        self._expanded = reduce(lambda x, y: x * y, sum_fracs)
         return self._expanded
 
     def get_unique_vars(self, var_type):
@@ -323,11 +342,15 @@ class Product(Expr):
         return var
 
     def get_var_occurrences(self):
-        """Determine the number of times all variables occurs in the expression.
-        Returns a dictionary of variables and the number of times they occur."""
-        # TODO: The product should be expanded at this stage, should we check
-        # this?
-        # Create dictionary and count number of occurrences of each variable.
+        """Determine the number of times all variables occurs in the
+        expression.  Returns a dictionary of variables and the number
+        of times they occur.
+
+        """
+        # TODO: The product should be expanded at this stage, should
+        # we check this?
+        # Create dictionary and count number of occurrences of each
+        # variable.
         d = {}
         for v in self.vrs:
             if v in d:
@@ -338,9 +361,10 @@ class Product(Expr):
 
     def get_vrs(self):
         "Return all 'real' variables."
-        # A product should only have one float value after initialisation.
+        # A product should only have one float value after
+        # initialisation.
         # TODO: Use this knowledge directly in other classes?
-        if self.vrs[0]._prec == 0: # float
+        if self.vrs[0]._prec == 0:  # float
             return tuple(self.vrs[1:])
         return tuple(self.vrs)
 
@@ -353,29 +377,30 @@ class Product(Expr):
         for v in self.vrs:
             op += v.ops()
 
-        # Subtract 1, if the first member is -1 i.e., -1*x*y -> x*y is only 1 op.
+        # Subtract 1, if the first member is -1 i.e., -1*x*y -> x*y is
+        # only 1 op.
         if self.vrs[0]._prec == 0 and self.vrs[0].val == -1.0:
             op -= 1
         return op
 
     def reduce_ops(self):
         "Reduce the number of operations to evaluate the product."
-        # It's not possible to reduce a product if it is already expanded and
-        # it should be at this stage.
-        # TODO: Is it safe to return self.expand().reduce_ops() if product is
-        # not expanded? And do we want to?
-#        if self._expanded:
-#            return self._expanded
-#        error("Product must be expanded first before we can reduce the number of operations.")
-        # TODO: This should crash if it goes wrong (the above is more correct but slower).
+        # It's not possible to reduce a product if it is already
+        # expanded and it should be at this stage.
+        # TODO: Is it safe to return self.expand().reduce_ops() if
+        # product is not expanded? And do we want to?
+
+        # TODO: This should crash if it goes wrong
         return self._expanded
 
     def reduce_vartype(self, var_type):
-        """Reduce expression with given var_type. It returns a tuple
-        (found, remain), where 'found' is an expression that only has variables
-        of type == var_type. If no variables are found, found=(). The 'remain'
-        part contains the leftover after division by 'found' such that:
-        self = found*remain."""
+        """Reduce expression with given var_type. It returns a tuple (found,
+        remain), where 'found' is an expression that only has
+        variables of type == var_type. If no variables are found,
+        found=(). The 'remain' part contains the leftover after
+        division by 'found' such that: self = found*remain.
+
+        """
         # Sort variables according to type.
         found = []
         remains = []
@@ -405,10 +430,3 @@ class Product(Expr):
 
         # Return whatever we found.
         return [(found, remains)]
-
-# FFC quadrature modules.
-from .floatvalue import FloatValue
-from .symbol     import Symbol
-from .sumobj    import Sum
-from .fraction   import Fraction
-
