@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright (C) 2013 Kristian B. Oelgaard
 #
 # This file is part of FFC.
@@ -16,16 +17,17 @@
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Marie E. Rognes, 2013
-# Modified by Martin Alnaes, 2013-2014
+# Modified by Martin Sandve Alnæs, 2013-2014
 
 from ufl.utils.sorting import sorted_by_key
 
 # FFC modules
-from ffc.log import info, error, warning
+from ffc.log import info, error
 from ffc.cpp import format
-from ffc.quadrature.symbolics import optimise_code, BASIS, IP, GEO, CONST
+from ffc.quadrature.symbolics import optimise_code, BASIS, IP, GEO
 from ffc.quadrature.symbolics import create_product, create_sum, create_symbol, create_fraction
 
+
 def optimize_integral_ir(ir, parameters):
     "Compute optimized intermediate representation of integral."
 
@@ -40,11 +42,11 @@ def optimize_integral_ir(ir, parameters):
     if parameters["optimisation"]:
 
         # Get parameters
-        integrals      = ir["trans_integrals"]
-        integral_type  = ir["integral_type"]
-        num_facets     = ir["num_facets"]
-        num_vertices   = ir["num_vertices"]
-        geo_consts     = ir["geo_consts"]
+        integrals = ir["trans_integrals"]
+        integral_type = ir["integral_type"]
+        num_facets = ir["num_facets"]
+        num_vertices = ir["num_vertices"]
+        geo_consts = ir["geo_consts"]
         psi_tables_map = ir["psi_tables_map"]
 
         # Optimize based on integral type
@@ -66,7 +68,7 @@ def optimize_integral_ir(ir, parameters):
                 for j in range(num_facets):
                     info("Optimising expressions for facet integral (%d, %d)" % (i, j))
                     if parameters["optimisation"] in ("precompute_ip_const", "precompute_basis_const"):
-                        _precompute_expressions(integrals[i][j], geo_consts,parameters["optimisation"])
+                        _precompute_expressions(integrals[i][j], geo_consts, parameters["optimisation"])
                     else:
                         _simplify_expression(integrals[i][j], geo_consts, psi_tables_map)
         elif integral_type == "vertex":
@@ -81,6 +83,7 @@ def optimize_integral_ir(ir, parameters):
 
     return ir
 
+
 def _simplify_expression(integral, geo_consts, psi_tables_map):
     for points, terms, functions, ip_consts, coordinate, conditionals in integral:
         # NOTE: sorted is needed to pass the regression tests on the buildbots
@@ -105,6 +108,7 @@ def _simplify_expression(integral, geo_consts, psi_tables_map):
             terms[loop][0][2] = psi_tables
             terms[loop][1] = new_entry_vals
 
+
 def _precompute_expressions(integral, geo_consts, optimisation):
     for points, terms, functions, ip_consts, coordinate, conditionals in integral:
         for loop, (data, entry_vals) in sorted_by_key(terms):
@@ -117,6 +121,7 @@ def _precompute_expressions(integral, geo_consts, optimisation):
                     new_entry_vals.append((entry, value, value.ops()))
             terms[loop][1] = new_entry_vals
 
+
 def _extract_variables(val, basis_consts, ip_consts, geo_consts, t_set, optimisation):
     f_G = format["geometry constant"]
     f_I = format["ip constant"]
@@ -217,7 +222,7 @@ def _extract_variables(val, basis_consts, ip_consts, geo_consts, t_set, optimisa
 #            new_sum = create_sum(vrs)
 #            if new_sum.t == BASIS:
 #                return new_sum
-##                return _reduce_expression(new_sum, [], basis_consts, f_B, True)
+# return _reduce_expression(new_sum, [], basis_consts, f_B, True)
 #            elif new_sum.t == IP:
 #                return _reduce_expression(new_sum, [], ip_consts, f_I, True)
 #            elif new_sum.t == GEO:
@@ -226,6 +231,7 @@ def _extract_variables(val, basis_consts, ip_consts, geo_consts, t_set, optimisa
 #    else:
 #        error("Must have product or sum here: %s" % repr(new_val))
 
+
 def _reduce_expression(expr, symbols, const_dict, f_name, use_expr_type=False):
     if use_expr_type:
         if expr not in const_dict:
