@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 "This file implements a class to represent a sum."
 
 # Copyright (C) 2009-2010 Kristian B. Oelgaard
@@ -17,6 +18,8 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 
+import six
+
 from ufl.utils.sorting import sorted_by_key
 
 # FFC modules.
@@ -29,12 +32,13 @@ from .symbolics import create_product
 from .symbolics import create_sum
 from .symbolics import create_fraction
 from .expr import Expr
-import six
 
-#global ind
-#ind = ""
+from .floatvalue import FloatValue
+
+
 class Sum(Expr):
     __slots__ = ("vrs", "_expanded", "_reduced")
+
     def __init__(self, variables):
         """Initialise a Sum object, it derives from Expr and contains the
         additional variables:
@@ -65,15 +69,15 @@ class Sum(Expr):
                 # Skip zero terms.
                 if abs(var.val) < EPS:
                     continue
-                elif var._prec == 0: # float
+                elif var._prec == 0:  # float
                     float_val += var.val
                     continue
-                elif var._prec == 3: # sum
+                elif var._prec == 3:  # sum
                     # Loop and handle variables of nested sum.
                     for v in var.vrs:
                         if abs(v.val) < EPS:
                             continue
-                        elif v._prec == 0: # float
+                        elif v._prec == 0:  # float
                             float_val += v.val
                             continue
                         self.vrs.append(v)
@@ -97,7 +101,6 @@ class Sum(Expr):
         # Type is equal to the smallest type in both lists.
         self.t = min([v.t for v in self.vrs])
 
-
         # Sort variables, (for representation).
         self.vrs.sort()
 
@@ -141,7 +144,7 @@ class Sum(Expr):
         # NOTE: We expect expanded sub-expressions with no nested operators.
         # Create list of new products using the '*' operator
         # TODO: Is this efficient?
-        new_prods = [v*other for v in self.vrs]
+        new_prods = [v * other for v in self.vrs]
 
         # Remove zero valued terms.
         # TODO: Can this still happen?
@@ -173,13 +176,13 @@ class Sum(Expr):
         # NOTE: Expect that other is expanded i.e., x + x -> 2*x which can be handled.
         # TODO: Fix (1 + y) / (x + x*y) -> 1 / x
         # Will this be handled when reducing operations on a fraction?
-        if other._prec == 3: # sum
+        if other._prec == 3:  # sum
             return create_fraction(self, other)
 
         # NOTE: We expect expanded sub-expressions with no nested operators.
         # Create list of new products using the '*' operator.
         # TODO: Is this efficient?
-        new_fracs = [v/other for v in self.vrs]
+        new_fracs = [v / other for v in self.vrs]
 
         # Remove zero valued terms.
         # TODO: Can this still happen?
@@ -206,35 +209,36 @@ class Sum(Expr):
 
         # TODO: This function might need some optimisation.
 
-        # Sort variables into symbols, products and fractions (add floats
-        # directly to new list, will be handled later). Add fractions if
-        # possible else add to list.
+        # Sort variables into symbols, products and fractions (add
+        # floats directly to new list, will be handled later). Add
+        # fractions if possible else add to list.
         new_variables = []
         syms = []
         prods = []
-        frac_groups = {}
-        # TODO: Rather than using '+', would it be more efficient to collect
-        # the terms first?
+        # TODO: Rather than using '+', would it be more efficient to
+        # collect the terms first?
         for var in self.vrs:
             exp = var.expand()
-            # TODO: Should we also group fractions, or put this in a separate function?
-            if exp._prec in (0, 4): # float or frac
+            # TODO: Should we also group fractions, or put this in a
+            # separate function?
+            if exp._prec in (0, 4):  # float or frac
                 new_variables.append(exp)
-            elif exp._prec == 1: # sym
+            elif exp._prec == 1:  # sym
                 syms.append(exp)
-            elif exp._prec == 2: # prod
+            elif exp._prec == 2:  # prod
                 prods.append(exp)
-            elif exp._prec == 3: # sum
+            elif exp._prec == 3:  # sum
                 for v in exp.vrs:
-                    if v._prec in (0, 4): # float or frac
+                    if v._prec in (0, 4):  # float or frac
                         new_variables.append(v)
-                    elif v._prec == 1: # sym
+                    elif v._prec == 1:  # sym
                         syms.append(v)
-                    elif v._prec == 2: # prod
+                    elif v._prec == 2:  # prod
                         prods.append(v)
 
-        # Sort all variables in groups: [2*x, -7*x], [(x + y), (2*x + 4*y)] etc.
-        # First handle product in order to add symbols if possible.
+        # Sort all variables in groups: [2*x, -7*x], [(x + y), (2*x +
+        # 4*y)] etc.  First handle product in order to add symbols if
+        # possible.
         prod_groups = {}
         for v in prods:
             if v.get_vrs() in prod_groups:
@@ -256,20 +260,19 @@ class Sum(Expr):
                 sym_groups[v] = v
 
         # Loop groups and add to new variable list.
-        for k,v in sorted_by_key(sym_groups):
+        for k, v in sorted_by_key(sym_groups):
             new_variables.append(v)
-        for k,v in sorted_by_key(prod_groups):
+        for k, v in sorted_by_key(prod_groups):
             new_variables.append(v)
-#        for k,v in frac_groups.iteritems():
-#            new_variables.append(v)
-#            append(v)
 
         if len(new_variables) > 1:
-            # Return new sum (will remove multiple instances of floats during construction).
+            # Return new sum (will remove multiple instances of floats
+            # during construction).
             self._expanded = create_sum(sorted(new_variables))
             return self._expanded
         elif new_variables:
-            # If we just have one variable left, return it since it is already expanded.
+            # If we just have one variable left, return it since it is
+            # already expanded.
             self._expanded = new_variables[0]
             return self._expanded
         error("Where did the variables go?")
@@ -283,10 +286,14 @@ class Sum(Expr):
         return var
 
     def get_var_occurrences(self):
-        """Determine the number of minimum number of times all variables occurs
-        in the expression. Returns a dictionary of variables and the number of
-        times they occur. x*x + x returns {x:1}, x + y returns {}."""
-        # NOTE: This function is only used if the numerator of a Fraction is a Sum.
+        """Determine the number of minimum number of times all variables
+        occurs in the expression. Returns a dictionary of variables
+        and the number of times they occur. x*x + x returns {x:1}, x +
+        y returns {}.
+
+        """
+        # NOTE: This function is only used if the numerator of a
+        # Fraction is a Sum.
 
         # Get occurrences in first expression.
         d0 = self.vrs[0].get_var_occurrences()
@@ -295,9 +302,10 @@ class Sum(Expr):
             d = var.get_var_occurrences()
             # Delete those variables in d0 that are not in d.
             for k, v in list(d0.items()):
-                if not k in d:
+                if k not in d:
                     del d0[k]
-            # Set the number of occurrences equal to the smallest number.
+            # Set the number of occurrences equal to the smallest
+            # number.
             for k, v in sorted_by_key(d):
                 if k in d0:
                     d0[k] = min(d0[k], v)
@@ -305,7 +313,8 @@ class Sum(Expr):
 
     def ops(self):
         "Return number of operations to compute value of sum."
-        # Subtract one operation as it only takes n-1 ops to sum n members.
+        # Subtract one operation as it only takes n-1 ops to sum n
+        # members.
         op = -1
 
         # Add the number of operations from sub-expressions.
@@ -316,9 +325,6 @@ class Sum(Expr):
 
     def reduce_ops(self):
         "Reduce the number of operations needed to evaluate the sum."
-#        global ind
-#        ind += " "
-#        print "\n%sreduce_ops, start" % ind
 
         if self._reduced:
             return self._reduced
@@ -327,130 +333,108 @@ class Sum(Expr):
 
         # TODO: The entire function looks expensive, can it be optimised?
 
-        # TODO: It is not necessary to create a new Sum if we do not have more
-        # than one Fraction.
+        # TODO: It is not necessary to create a new Sum if we do not
+        # have more than one Fraction.
+
         # First group all fractions in the sum.
         new_sum = _group_fractions(self)
-        if new_sum._prec != 3: # sum
+        if new_sum._prec != 3:  # sum
             self._reduced = new_sum.reduce_ops()
             return self._reduced
-        # Loop all variables of the sum and collect the number of common
-        # variables that can be factored out.
+        # Loop all variables of the sum and collect the number of
+        # common variables that can be factored out.
         common_vars = {}
         for var in new_sum.vrs:
-            # Get dictonary of occurrences and add the variable and the number
-            # of occurrences to common dictionary.
+            # Get dictonary of occurrences and add the variable and
+            # the number of occurrences to common dictionary.
             for k, v in sorted_by_key(var.get_var_occurrences()):
-#                print
-#                print ind + "var: ", var
-#                print ind + "k: ", k
-#                print ind + "v: ", v
                 if k in common_vars:
                     common_vars[k].append((v, var))
                 else:
                     common_vars[k] = [(v, var)]
-#        print
-#        print "common vars: "
-#        for k,v in common_vars.items():
-#            print "k: ", k
-#            print "v: ", v
-#        print
-        # Determine the maximum reduction for each variable
-        # sorted as: {(x*x*y, x*y*z, 2*y):[2, [y]]}.
+
+        # Determine the maximum reduction for each variable sorted as:
+        # {(x*x*y, x*y*z, 2*y):[2, [y]]}.
         terms_reductions = {}
         for k, v in sorted_by_key(common_vars):
-#            print
-#            print ind + "k: ", k
-#            print ind + "v: ", v
-            # If the number of expressions that can be reduced is only one
-            # there is nothing to be done.
+            # If the number of expressions that can be reduced is only
+            # one there is nothing to be done.
             if len(v) > 1:
-                # TODO: Is there a better way to compute the reduction gain
-                # and the number of occurrences we should remove?
+                # TODO: Is there a better way to compute the reduction
+                # gain and the number of occurrences we should remove?
 
-                # Get the list of number of occurences of 'k' in expressions
-                # in 'v'.
+                # Get the list of number of occurences of 'k' in
+                # expressions in 'v'.
                 occurrences = [t[0] for t in v]
 
-                # Determine the favorable number of occurences and an estimate
-                # of the maximum reduction for current variable.
+                # Determine the favorable number of occurences and an
+                # estimate of the maximum reduction for current
+                # variable.
                 fav_occur = 0
                 reduc = 0
                 for i in set(occurrences):
-                    # Get number of terms that has a number of occcurences equal
-                    # to or higher than the current number.
+                    # Get number of terms that has a number of
+                    # occcurences equal to or higher than the current
+                    # number.
                     num_terms = len([o for o in occurrences if o >= i])
 
                     # An estimate of the reduction in operations is:
                     # (number_of_terms - 1) * number_occurrences.
-                    new_reduc = (num_terms-1)*i
+                    new_reduc = (num_terms - 1) * i
                     if new_reduc > reduc:
                         reduc = new_reduc
                         fav_occur = i
 
-                # Extract the terms of v where the number of occurrences is
-                # equal to or higher than the most favorable number of occurrences.
+                # Extract the terms of v where the number of
+                # occurrences is equal to or higher than the most
+                # favorable number of occurrences.
                 terms = sorted([t[1] for t in v if t[0] >= fav_occur])
 
-                # We need to reduce the expression with the favorable number of
-                # occurrences of the current variable.
-                red_vars = [k]*fav_occur
+                # We need to reduce the expression with the favorable
+                # number of occurrences of the current variable.
+                red_vars = [k] * fav_occur
 
-                # If the list of terms is already present in the dictionary,
-                # add the reduction count and the variables.
+                # If the list of terms is already present in the
+                # dictionary, add the reduction count and the
+                # variables.
                 if tuple(terms) in terms_reductions:
                     terms_reductions[tuple(terms)][0] += reduc
                     terms_reductions[tuple(terms)][1] += red_vars
                 else:
                     terms_reductions[tuple(terms)] = [reduc, red_vars]
-#        print "\nterms_reductions: "
-#        for k,v in terms_reductions.items():
-#            print "k: ", create_sum(k)
-#            print "v: ", v
-#        print "red: self: ", self
+
         if terms_reductions:
             # Invert dictionary of terms.
-            reductions_terms = dict([((v[0], tuple(v[1])), k) for k, v in six.iteritems(terms_reductions)])
+            reductions_terms = dict([((v[0], tuple(v[1])), k) for k,
+                                     v in six.iteritems(terms_reductions)])
 
-            # Create a sorted list of those variables that give the highest
-            # reduction.
-            sorted_reduc_var = sorted(six.iterkeys(reductions_terms), reverse=True)
-#            sorted_reduc_var = [k for k, v in six.iteritems(reductions_terms)]
-#            print
-#            print ind + "raw"
-#            for k in sorted_reduc_var:
-#                print ind, k[0], k[1]
-#            sorted_reduc_var.sort()
-#            sorted_reduc_var.sort(lambda x, y: cmp(x[0], y[0]))
-#            sorted_reduc_var.reverse()
-#            print ind + "sorted"
-#            for k in sorted_reduc_var:
-#                print ind, k[0], k[1]
+            # Create a sorted list of those variables that give the
+            # highest reduction.
+            sorted_reduc_var = sorted(six.iterkeys(reductions_terms),
+                                      reverse=True)
 
-            # Create a new dictionary of terms that should be reduced, if some
-            # terms overlap, only pick the one which give the highest reduction to
-            # ensure that a*x*x + b*x*x + x*x*y + 2*y -> x*x*(a + b + y) + 2*y NOT
-            # x*x*(a + b) + y*(2 + x*x).
+            # Create a new dictionary of terms that should be reduced,
+            # if some terms overlap, only pick the one which give the
+            # highest reduction to ensure that a*x*x + b*x*x + x*x*y +
+            # 2*y -> x*x*(a + b + y) + 2*y NOT x*x*(a + b) + y*(2 +
+            # x*x).
             reduction_vars = {}
             rejections = {}
             for var in sorted_reduc_var:
                 terms = reductions_terms[var]
-                if _overlap(terms, reduction_vars) or _overlap(terms, rejections):
+                if _overlap(terms, reduction_vars) or _overlap(terms,
+                                                               rejections):
                     rejections[var[1]] = terms
                 else:
                     reduction_vars[var[1]] = terms
 
-#            print "\nreduction_vars: "
-#            for k,v in reduction_vars.items():
-#                print "k: ", k
-#                print "v: ", v
-
             # Reduce each set of terms with appropriate variables.
             all_reduced_terms = []
             reduced_expressions = []
             for reduc_var, terms in sorted(six.iteritems(reduction_vars)):
 
-                # Add current terms to list of all variables that have been reduced.
+                # Add current terms to list of all variables that have
+                # been reduced.
                 all_reduced_terms += list(terms)
 
                 # Create variable that we will use to reduce the terms.
@@ -467,23 +451,27 @@ class Sum(Expr):
                 reduced_expr = None
                 if len(reduced_terms) > 1:
                     # Try to reduce the reduced terms further.
-                    reduced_expr = create_product([reduction_var, create_sum(reduced_terms).reduce_ops()])
+                    reduced_expr = create_product([reduction_var,
+                                                   create_sum(reduced_terms).reduce_ops()])
                 else:
-                    reduced_expr = create_product(reduction_var, reduced_terms[0])
+                    reduced_expr = create_product(reduction_var,
+                                                  reduced_terms[0])
 
-                # Add reduced expression to list of reduced expressions.
+                # Add reduced expression to list of reduced
+                # expressions.
                 reduced_expressions.append(reduced_expr)
 
             # Create list of terms that should not be reduced.
             dont_reduce_terms = []
             for v in new_sum.vrs:
-                if not v in all_reduced_terms:
+                if v not in all_reduced_terms:
                     dont_reduce_terms.append(v)
 
             # Create expression from terms that was not reduced.
             not_reduced_expr = None
             if dont_reduce_terms and len(dont_reduce_terms) > 1:
-                # Try to reduce the remaining terms that were not reduced at first.
+                # Try to reduce the remaining terms that were not
+                # reduced at first.
                 not_reduced_expr = create_sum(dont_reduce_terms).reduce_ops()
             elif dont_reduce_terms:
                 not_reduced_expr = dont_reduce_terms[0]
@@ -495,42 +483,26 @@ class Sum(Expr):
                 self._reduced = create_sum(reduced_expressions)
             else:
                 self._reduced = reduced_expressions[0]
-#            # NOTE: Only switch on for debugging.
-#            if not self._reduced.expand() == self.expand():
-#                print reduced_expressions[0]
-#                print reduced_expressions[0].expand()
-#                print "self: ", self
-#                print "red:  ", repr(self._reduced)
-#                print "self.exp: ", self.expand()
-#                print "red.exp:  ", self._reduced.expand()
-#                error("Reduced expression is not equal to original expression.")
+
             return self._reduced
 
-        # Return self if we don't have any variables for which we can reduce
-        # the sum.
+        # Return self if we don't have any variables for which we can
+        # reduce the sum.
         self._reduced = self
         return self._reduced
 
     def reduce_vartype(self, var_type):
         """Reduce expression with given var_type. It returns a list of tuples
-        [(found, remain)], where 'found' is an expression that only has variables
-        of type == var_type. If no variables are found, found=(). The 'remain'
-        part contains the leftover after division by 'found' such that:
-        self = Sum([f*r for f,r in self.reduce_vartype(Type)])."""
+        [(found, remain)], where 'found' is an expression that only
+        has variables of type == var_type. If no variables are found,
+        found=(). The 'remain' part contains the leftover after
+        division by 'found' such that: self = Sum([f*r for f,r in
+        self.reduce_vartype(Type)]).
+
+        """
         found = {}
-#        print "\nself: ", self
         # Loop members and reduce them by vartype.
         for v in self.vrs:
-#            print "v: ", v
-#            print "red: ", v.reduce_vartype(var_type)
-#            red = v.reduce_vartype(var_type)
-#            f, r = v.reduce_vartype(var_type)
-#            print "len red: ", len(red)
-#            print "red: ", red
-#            if len(red) == 2:
-#                f, r = red
-#            else:
-#                raise RuntimeError
             for f, r in v.reduce_vartype(var_type):
                 if f in found:
                     found[f].append(r)
@@ -542,13 +514,13 @@ class Sum(Expr):
         for f, r in sorted_by_key(found):
             if len(r) > 1:
                 # Use expand to group expressions.
-#                r = create_sum(r).expand()
                 r = create_sum(r)
             elif r:
                 r = r.pop()
             returns.append((f, r))
         return sorted(returns)
 
+
 def _overlap(l, d):
     "Check if a member in list l is in the value (list) of dictionary d."
     for m in l:
@@ -557,16 +529,17 @@ def _overlap(l, d):
                 return True
     return False
 
+
 def _group_fractions(expr):
     "Group Fractions in a Sum: 2/x + y/x -> (2 + y)/x."
-    if expr._prec != 3: # sum
+    if expr._prec != 3:  # sum
         return expr
 
     # Loop variables and group those with common denominator.
     not_frac = []
     fracs = {}
     for v in expr.vrs:
-        if v._prec == 4: # frac
+        if v._prec == 4:  # frac
             if v.denom in fracs:
                 fracs[v.denom][1].append(v.num)
                 fracs[v.denom][0] += 1
@@ -577,11 +550,12 @@ def _group_fractions(expr):
     if not fracs:
         return expr
 
-    # Loop all fractions and create new ones using an appropriate numerator.
+    # Loop all fractions and create new ones using an appropriate
+    # numerator.
     for k, v in sorted(six.iteritems(fracs)):
         if v[0] > 1:
-            # TODO: Is it possible to avoid expanding the Sum?
-            # I think we have to because x/a + 2*x/a -> 3*x/a.
+            # TODO: Is it possible to avoid expanding the Sum?  I
+            # think we have to because x/a + 2*x/a -> 3*x/a.
             not_frac.append(create_fraction(create_sum(v[1]).expand(), k))
         else:
             not_frac.append(v[2])
@@ -590,8 +564,3 @@ def _group_fractions(expr):
     if len(not_frac) > 1:
         return create_sum(not_frac)
     return not_frac[0]
-
-from .floatvalue import FloatValue
-from .symbol     import Symbol
-from .product    import Product
-from .fraction   import Fraction
