@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 "This module implements efficient integration of monomial forms."
 
 # Copyright (C) 2004-2011 Anders Logg
@@ -32,7 +33,6 @@ import time
 from ffc.log import info, debug, error
 from ffc.fiatinterface import create_element
 from ffc.fiatinterface import map_facet_points
-from ffc.quadrature_schemes import create_quadrature
 from ffc.representationutils import create_quadrature_points_and_weights
 
 # FFC tensor representation modules
@@ -40,6 +40,7 @@ from .multiindex import build_indices
 from .monomialextraction import MonomialException
 from .monomialtransformation import MonomialIndex
 
+
 def integrate(monomial,
               integral_type,
               facet0, facet1,
@@ -55,7 +56,8 @@ def integrate(monomial,
     tic = time.time()
 
     # Initialize quadrature points and weights
-    (points, weights) = create_quadrature_points_and_weights(integral_type, cell, quadrature_degree, quadrature_rule)
+    (points, weights) = create_quadrature_points_and_weights(
+        integral_type, cell, quadrature_degree, quadrature_rule)
 
     # Initialize quadrature table for basis functions
     table = _init_table(monomial.arguments,
@@ -64,8 +66,8 @@ def integrate(monomial,
                         facet0, facet1)
 
     # Compute table Psi for each factor
-    psis = [_compute_psi(v, table, len(points)) \
-                for v in monomial.arguments]
+    psis = [_compute_psi(v, table, len(points))
+            for v in monomial.arguments]
 
     # Compute product of all Psis
     A0 = _compute_product(psis, monomial.float_value * weights)
@@ -78,6 +80,7 @@ def integrate(monomial,
 
     return A0
 
+
 def _init_table(arguments, integral_type, points, facet0, facet1):
     """Initialize table of basis functions and their derivatives at
     the given quadrature points for each element."""
@@ -109,6 +112,7 @@ def _init_table(arguments, integral_type, points, facet0, facet1):
 
     return table
 
+
 def _compute_psi(v, table, num_points):
     "Compute the table Psi for the given basis function v."
 
@@ -131,7 +135,7 @@ def _compute_psi(v, table, num_points):
     tdim = v.element.cell().topological_dimension()
 
     # Get indices and shapes for components
-    if len(v.components) ==  0:
+    if len(v.components) == 0:
         cindex = []
         cshape = []
     elif len(v.components) == 1:
@@ -153,7 +157,7 @@ def _compute_psi(v, table, num_points):
     shapes = cshape + dshape + vshape + [num_points]
 
     # Initialize tensor Psi: component, derivatives, basis function, points
-    Psi = numpy.zeros(shapes, dtype = numpy.float)
+    Psi = numpy.zeros(shapes, dtype=numpy.float)
 
     # Iterate over derivative indices
     dlists = build_indices([index.index_range for index in dindex]) or [[]]
@@ -181,8 +185,8 @@ def _compute_psi(v, table, num_points):
     # Remove fixed indices
     for i in range(num_indices[0]):
         Psi = Psi[0, ...]
-    indices = [index for index in indices \
-                   if not index.index_type == MonomialIndex.FIXED]
+    indices = [index for index in indices
+               if not index.index_type == MonomialIndex.FIXED]
 
     # Put quadrature points first
     rank = Psi.ndim
@@ -193,6 +197,7 @@ def _compute_psi(v, table, num_points):
 
     return (Psi, indices, bpart)
 
+
 def _compute_product(psis, weights):
     "Compute special product of list of Psis."
 
@@ -203,7 +208,7 @@ def _compute_product(psis, weights):
 
     # Initialize zero reference tensor (will be rearranged later)
     (shape, indices) = _compute_shape(psis)
-    A0 = numpy.zeros(shape, dtype= numpy.float)
+    A0 = numpy.zeros(shape, dtype=numpy.float)
 
     # Initialize list of internal multiindices
     bshape = _compute_internal_shape(psis)
@@ -217,7 +222,7 @@ def _compute_product(psis, weights):
             # Compute outer products of subtables for current (q, b)
             B = weights[q]
             for (Psi, index, bpart) in psis:
-                B = numpy.multiply.outer(B, Psi[ tuple([q] + [b[i] for i in bpart])])
+                B = numpy.multiply.outer(B, Psi[tuple([q] + [b[i] for i in bpart])])
 
             # Add product to reference tensor
             numpy.add(A0, B, A0)
@@ -228,20 +233,22 @@ def _compute_product(psis, weights):
 
     return A0
 
+
 def _compute_rearrangement(indices):
     """
     Compute rearrangement tuple for given list of Indices, so that the
     tuple reorders the given list of Indices with fixed, primary,
     secondary and internal Indices in rising order.
     """
-    fixed     = _find_indices(indices, MonomialIndex.FIXED)
-    internal  = _find_indices(indices, MonomialIndex.INTERNAL)
-    primary   = _find_indices(indices, MonomialIndex.PRIMARY)
+    fixed = _find_indices(indices, MonomialIndex.FIXED)
+    internal = _find_indices(indices, MonomialIndex.INTERNAL)
+    primary = _find_indices(indices, MonomialIndex.PRIMARY)
     secondary = _find_indices(indices, MonomialIndex.SECONDARY)
     assert len(fixed + internal + primary + secondary) == len(indices)
-    return (tuple(fixed + internal + primary + secondary), \
+    return (tuple(fixed + internal + primary + secondary),
             (len(fixed), len(internal), len(primary), len(secondary)))
 
+
 def _compute_shape(psis):
     "Compute shape of reference tensor from given list of tables."
     shape, indices = [], []
@@ -251,6 +258,7 @@ def _compute_shape(psis):
         indices += index[num_internal:]
     return (shape, indices)
 
+
 def _compute_internal_shape(psis):
     """
     Compute shape for internal indices from given list of tables.
@@ -259,7 +267,8 @@ def _compute_internal_shape(psis):
     """
     # First find the number of different internal indices (check maximum)
     bs = [b for (Psi, index, bpart) in psis for b in bpart]
-    if len(bs) == 0: return []
+    if len(bs) == 0:
+        return []
     bmax = max(bs)
     # Find the dimension for each internal index
     bshape = [0 for i in range(bmax + 1)]
@@ -271,12 +280,14 @@ def _compute_internal_shape(psis):
         error("Unable to compute the shape for each internal index.")
     return bshape
 
+
 def _find_indices(indices, index_type):
     "Return sorted list of positions for given index type."
     pos = [i for i in range(len(indices)) if indices[i].index_type == index_type]
     val = [indices[i].index_id for i in range(len(indices)) if indices[i].index_type == index_type]
     return [pos[i] for i in numpy.argsort(val)]
 
+
 def _multiindex_to_tuple(dindex, cell_dimension):
     """
     Compute lookup tuple from given derivative multiindex. Necessary
