@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """This module implements the representation of a multilinear form as
 a sum of tensor contractions.
 
@@ -25,7 +26,7 @@ might be (re-)implemented in a future version of FFC
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Kristian B. Oelgaard, 2010.
-# Modified by Martin Alnaes, 2013
+# Modified by Martin Sandve Alnæs, 2013
 
 # FFC modules
 from ffc.log import info, error
@@ -38,10 +39,12 @@ from ffc.tensor.referencetensor import ReferenceTensor
 from ffc.tensor.geometrytensor import GeometryTensor
 from ffc.tensor.tensorreordering import reorder_entries
 
+
 def compute_integral_ir(itg_data,
                         form_data,
                         form_id,
                         element_numbers,
+                        classnames,
                         parameters):
     "Compute intermediate represention of integral."
 
@@ -65,11 +68,11 @@ def compute_integral_ir(itg_data,
 
     # Helper to simplify code below
     compute_terms = lambda i, j: _compute_terms(monomial_form,
-                                           i, j,
-                                           integral_type,
-                                           quadrature_degree,
-                                           quadrature_rule,
-                                           cell)
+                                                i, j,
+                                                integral_type,
+                                                quadrature_degree,
+                                                quadrature_rule,
+                                                cell)
 
     # Compute representation of cell tensor
     if integral_type == "cell":
@@ -96,6 +99,7 @@ def compute_integral_ir(itg_data,
 
     return ir
 
+
 def _compute_terms(monomial_form,
                    facet0, facet1,
                    integral_type,
