@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+# -*- coding: utf-8 -*-
 """This script compiles and verifies the output for all form files
 found in the 'demo' directory. The verification is performed in two
 steps. First, the generated code is compared with stored references.
@@ -28,7 +28,7 @@ option --bench.
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
-# Modified by Martin Alnaes, 2013-2015
+# Modified by Martin Sandve Alnæs, 2013-2016
 # Modified by Johannes Ring, 2013
 # Modified by Kristian B. Oelgaard, 2013
 # Modified by Garth N. Wells, 2014
@@ -36,63 +36,115 @@ option --bench.
 # FIXME: Need to add many more test cases. Quite a few DOLFIN forms
 # failed after the FFC tests passed.
 
-import os, sys, shutil, difflib, sysconfig
-from numpy import array, shape, abs, max, isnan
-from ffc.log import begin, end, info, info_red, info_green, info_blue
-from ufctest import generate_test_code
-from instant.output import get_status_output
+import os
+import sys
+import shutil
+import difflib
+import sysconfig
+import subprocess
 import time
+import logging
+from numpy import array, shape, abs, max, isnan
+import ffc
+from ffc.log import begin, end, info, info_red, info_green, info_blue
+from ffc.log import ffc_logger, ERROR, WARNING
+from ufl.log import ufl_logger
+from ufl.utils.py23 import as_native_str
+from ffc import get_ufc_cxx_flags
+from ffc.backends.ufc import get_include_path as get_ufc_include
+from ufctest import generate_test_code
 
-# Parameters
-debug = False # TODO: Can make this a cmdline argument, and start crashing programs in debugger automatically?
+# Parameters TODO: Can make this a cmdline argument, and start
+# crashing programs in debugger automatically?
+debug = False
 output_tolerance = 1.e-6
 demo_directory = "../../../../demo"
 bench_directory = "../../../../bench"
 
 # Global log file
-logfile = None
+logfile = "error.log"
+
+# Remove old log file
+if os.path.isfile(logfile):
+    os.remove(logfile)
+
+class GEFilter(object):
+    """Filter messages that are greater or equal to given log level"""
+    def __init__(self, level):
+        self.__level = level
+
+    def filter(self, record):
+        return record.levelno >= self.__level
+
+class LTFilter(object):
+    """Filter messages that are less than given log level"""
+    def __init__(self, level):
+        self.__level = level
+
+    def filter(self, record):
+        return record.levelno < self.__level
+
+# Filter out error messages from std output
+splitlevel = ERROR
+ffc_logger.get_handler().addFilter(LTFilter(splitlevel))
+ufl_logger.get_handler().addFilter(LTFilter(splitlevel))
+
+# Filter out error messages to log file
+file_handler = logging.FileHandler(logfile)
+file_handler.addFilter(GEFilter(splitlevel))
+ffc_logger.get_logger().addHandler(file_handler)
+ufl_logger.get_logger().addHandler(file_handler)
 
 # Extended quadrature tests (optimisations)
-ext_quad = [\
-"-r quadrature -O -feliminate_zeros",
-"-r quadrature -O -fsimplify_expressions",
-"-r quadrature -O -fprecompute_ip_const",
-"-r quadrature -O -fprecompute_basis_const",
-"-r quadrature -O -fprecompute_ip_const -feliminate_zeros",
-"-r quadrature -O -fprecompute_basis_const -feliminate_zeros",
+ext_quad = [
+    "-r quadrature -O -feliminate_zeros",
+    "-r quadrature -O -fsimplify_expressions",
+    "-r quadrature -O -fprecompute_ip_const",
+    "-r quadrature -O -fprecompute_basis_const",
+    "-r quadrature -O -fprecompute_ip_const -feliminate_zeros",
+    "-r quadrature -O -fprecompute_basis_const -feliminate_zeros",
 ]
 
-# Extended uflacs tests (to be extended with optimisation parameters later)
-ext_uflacs = [\
-"-r uflacs",
+# Extended uflacs tests (to be extended with optimisation parameters
+# later)
+ext_uflacs = [
+    "-r uflacs",
 ]
 
+known_uflacs_failures = set([
+    "CustomIntegral.ufl",
+    "CustomMixedIntegral.ufl",
+    "CustomVectorIntegral.ufl",
+])
+
 _command_timings = []
+
+
 def run_command(command):
     "Run command and collect errors in log file."
-    # Debugging:
-    #print "IN DIRECTORY:", os.path.abspath(os.curdir)
-    #print "RUNNING COMMAND:", command
-    t1 = time.time()
-    (status, output) = get_status_output(command)
-    t2 = time.time()
     global _command_timings
-    _command_timings.append((command, t2-t1))
-    if status == 0:
+
+    t1 = time.time()
+    try:
+        output = as_native_str(subprocess.check_output(command, shell=True))
+        t2 = time.time()
+        _command_timings.append((command, t2 - t1))
+        verbose = False  # FIXME: Set from --verbose
+        if verbose:
+            print(output)
         return True
-    global logfile
-    if logfile is None:
-        logfile = open("../../error.log", "w")
-    logfile.write(output + "\n")
-    print(output)
-    return False
+    except subprocess.CalledProcessError as e:
+        t2 = time.time()
+        _command_timings.append((command, t2 - t1))
+        log_error(e.output)
+        print(e.output)
+        return False
+
 
 def log_error(message):
     "Log error message."
-    global logfile
-    if logfile is None:
-        logfile = open("../../error.log", "w")
-    logfile.write(message + "\n")
+    ffc_logger.get_logger().error(message)
+
 
 def clean_output(output_directory):
     "Clean out old output directory"
@@ -100,7 +152,8 @@ def clean_output(output_directory):
         shutil.rmtree(output_directory)
     os.mkdir(output_directory)
 
-def generate_test_cases(bench, only_forms):
+
+def generate_test_cases(bench, only_forms, skip_forms):
     "Generate form files for all test cases."
 
     begin("Generating test cases")
@@ -112,7 +165,8 @@ def generate_test_cases(bench, only_forms):
         form_directory = demo_directory
 
     # Make list of form files
-    form_files = [f for f in os.listdir(form_directory) if f.endswith(".ufl")]
+    form_files = [f for f in os.listdir(form_directory)
+                  if f.endswith(".ufl") and f not in skip_forms]
     if only_forms:
         form_files = [f for f in form_files if f in only_forms]
     form_files.sort()
@@ -122,22 +176,27 @@ def generate_test_cases(bench, only_forms):
     info_green("Found %d form files" % len(form_files))
 
     # Generate form files for forms
-    info("Generating form files for extra forms: Not implemented")
+    #info("Generating form files for extra forms: Not implemented")
 
     # Generate form files for elements
-    if not bench:
+    if not (bench or only_forms):
         from elements import elements
-        info("Generating form files for extra elements (%d elements)" % len(elements))
+        info("Generating form files for extra elements (%d elements)"
+             % len(elements))
         for (i, element) in enumerate(elements):
-            open("X_Element%d.ufl" % i, "w").write("element = %s" % element)
+            with open("X_Element%d.ufl" % i, "w") as f:
+                f.write("element = %s" % element)
 
     end()
 
-def generate_code(args, only_forms):
+
+def generate_code(args, only_forms, skip_forms):
     "Generate code for all test cases."
+    global _command_timings
 
     # Get a list of all files
-    form_files = [f for f in os.listdir(".") if f.endswith(".ufl")]
+    form_files = [f for f in os.listdir(".")
+                  if f.endswith(".ufl") and f not in skip_forms]
     if only_forms:
         form_files = [f for f in form_files if f in only_forms]
     form_files.sort()
@@ -146,36 +205,47 @@ def generate_code(args, only_forms):
 
     # TODO: Parse additional options from .ufl file? I.e. grep for
     # some sort of tag like '#ffc: <flags>'.
-    special = {
-        "AdaptivePoisson.ufl": "-e",
-        }
+    special = {"AdaptivePoisson.ufl": "-e", }
 
     # Iterate over all files
     for f in form_files:
-        options = special.get(f, "")
+        options = [special.get(f, "")]
+        options.extend(args)
+        options.extend(["-f", "precision=8", "-fconvert_exceptions_to_warnings"])
+        options.append(f)
+        options = list(filter(None, options))
 
-        cmd = ("ffc %s %s -f precision=8 -fconvert_exceptions_to_warnings %s"
-               % (options, " ".join(args), f))
+        cmd = sys.executable + " -m ffc " + " ".join(options)
 
         # Generate code
-        ok = run_command(cmd)
+        t1 = time.time()
+        try:
+            ok = ffc.main(options)
+        except Exception as e:
+            log_error(e)
+            ok = 1
+            raise
+        finally:
+            t2 = time.time()
+            _command_timings.append((cmd, t2 - t1))
 
         # Check status
-        if ok:
+        if ok == 0:
             info_green("%s OK" % f)
         else:
             info_red("%s failed" % f)
 
     end()
 
+
 def validate_code(reference_dir):
     "Validate generated code against references."
 
     # Get a list of all files
-    header_files = [f for f in os.listdir(".") if f.endswith(".h")]
-    header_files.sort()
+    header_files = sorted([f for f in os.listdir(".") if f.endswith(".h")])
 
-    begin("Validating generated code (%d header files found)" % len(header_files))
+    begin("Validating generated code (%d header files found)"
+          % len(header_files))
 
     # Iterate over all files
     for f in header_files:
@@ -196,26 +266,19 @@ def validate_code(reference_dir):
             info_green("%s OK" % f)
         else:
             info_red("%s differs" % f)
-            diff = "\n".join([line for line in difflib.unified_diff(reference_code.split("\n"), generated_code.split("\n"))])
+            difflines = difflib.unified_diff(
+                reference_code.split("\n"),
+                generated_code.split("\n"))
+            diff = "\n".join(difflines)
             s = ("Code differs for %s, diff follows (reference first, generated second)"
                  % os.path.join(*reference_file.split(os.path.sep)[-3:]))
-            log_error("\n" + s + "\n" + len(s)*"-")
+            log_error("\n" + s + "\n" + len(s) * "-")
             log_error(diff)
 
     end()
 
-def build_programs(bench, permissive):
-    "Build test programs for all test cases."
-
-    # Get a list of all files
-    header_files = [f for f in os.listdir(".") if f.endswith(".h")]
-    header_files.sort()
-
-    begin("Building test programs (%d header files found)" % len(header_files))
-
-    # Get UFC flags
-    ufc_cflags = get_status_output("pkg-config --cflags ufc-1")[1].strip()
 
+def find_boost_cflags():
     # Get Boost dir (code copied from ufc/src/utils/python/ufc_utils/build.py)
     # Set a default directory for the boost installation
     if sys.platform == "darwin":
@@ -231,13 +294,15 @@ def build_programs(bench, permissive):
     boost_dir = os.getenv("BOOST_DIR", default)
     boost_is_found = False
     for inc_dir in ["", "include"]:
-        if os.path.isfile(os.path.join(boost_dir, inc_dir, "boost", "version.hpp")):
+        if os.path.isfile(os.path.join(boost_dir, inc_dir, "boost",
+                                       "version.hpp")):
             boost_inc_dir = os.path.join(boost_dir, inc_dir)
             break
     libdir_multiarch = "lib/" + sysconfig.get_config_vars().get("MULTIARCH", "")
     for lib_dir in ["", "lib", libdir_multiarch, "lib64"]:
         for ext in [".so", "-mt.so", ".dylib", "-mt.dylib"]:
-            _lib = os.path.join(boost_dir, lib_dir, "lib" + boost_math_tr1_lib + ext)
+            _lib = os.path.join(boost_dir, lib_dir, "lib" + boost_math_tr1_lib
+                                + ext)
             if os.path.isfile(_lib):
                 if "-mt" in _lib:
                     boost_math_tr1_lib += "-mt"
@@ -246,24 +311,44 @@ def build_programs(bench, permissive):
     if boost_inc_dir != "" and boost_lib_dir != "":
         boost_is_found = True
 
-    if not boost_is_found:
-        raise OSError("""The Boost library was not found.
+    if boost_is_found:
+        boost_cflags = " -I%s -L%s" % (boost_inc_dir, boost_lib_dir)
+        boost_linkflags = "-l%s" % boost_math_tr1_lib
+    else:
+        boost_cflags = ""
+        boost_linkflags = ""
+        info_red("""The Boost library was not found.
 If Boost is installed in a nonstandard location,
 set the environment variable BOOST_DIR.
+Forms using bessel functions will fail to build.
 """)
+    return boost_cflags, boost_linkflags
 
-    ufc_cflags += " -I%s -L%s" % (boost_inc_dir, boost_lib_dir)
+
+def build_programs(bench, permissive):
+    "Build test programs for all test cases."
+
+    # Get a list of all files
+    header_files = sorted([f for f in os.listdir(".") if f.endswith(".h")])
+
+    begin("Building test programs (%d header files found)" % len(header_files))
+
+    # Get UFC flags
+    ufc_cflags = "-I" + get_ufc_include() + " " + " ".join(get_ufc_cxx_flags())
+    boost_cflags, boost_linkflags = find_boost_cflags()
+    ufc_cflags += boost_cflags
+    linker_options = boost_linkflags
 
     # Set compiler options
-    compiler_options = "%s -Wall" % ufc_cflags
+    compiler_options = "%s -Wall " % ufc_cflags
     if not permissive:
         compiler_options += " -Werror -pedantic"
     if bench:
         info("Benchmarking activated")
         # Takes too long to build with -O2
-        #compiler_options += " -O2"
+        # compiler_options += " -O2"
         compiler_options += " -O3"
-        #compiler_options += " -O3 -fno-math-errno -march=native"
+        # compiler_options += " -O3 -fno-math-errno -march=native"
     if debug:
         info("Debugging activated")
         compiler_options += " -g -O0"
@@ -277,8 +362,8 @@ set the environment variable BOOST_DIR.
 
         # Compile test code
         prefix = f.split(".h")[0]
-        command = "g++ %s -o %s.bin %s.cpp -l%s" % \
-                  (compiler_options, prefix, prefix, boost_math_tr1_lib)
+        command = "g++ %s -o %s.bin %s.cpp %s" % \
+                  (compiler_options, prefix, prefix, linker_options)
         ok = run_command(command)
 
         # Check status
@@ -289,6 +374,7 @@ set the environment variable BOOST_DIR.
 
     end()
 
+
 def run_programs(bench):
     "Run generated programs."
 
@@ -296,8 +382,7 @@ def run_programs(bench):
     bench = 'b' if bench else ''
 
     # Get a list of all files
-    test_programs = [f for f in os.listdir(".") if f.endswith(".bin")]
-    test_programs.sort()
+    test_programs = sorted([f for f in os.listdir(".") if f.endswith(".bin")])
 
     begin("Running generated programs (%d programs found)" % len(test_programs))
 
@@ -316,13 +401,15 @@ def run_programs(bench):
 
     end()
 
+
 def validate_programs(reference_dir):
     "Validate generated programs against references."
 
     # Get a list of all files
     output_files = sorted(f for f in os.listdir(".") if f.endswith(".json"))
 
-    begin("Validating generated programs (%d .json program output files found)" % len(output_files))
+    begin("Validating generated programs (%d .json program output files found)"
+          % len(output_files))
 
     # Iterate over all files
     for fj in output_files:
@@ -332,7 +419,8 @@ def validate_programs(reference_dir):
             generated_json_output = open(fj).read()
             if "nan" in generated_json_output:
                 info_red("Found nan in generated json output, replacing with 999 to be able to parse as python dict.")
-                generated_json_output = generated_json_output.replace("nan", "999")
+                generated_json_output = generated_json_output.replace("nan",
+                                                                      "999")
         else:
             generated_json_output = "{}"
 
@@ -370,36 +458,53 @@ def validate_programs(reference_dir):
 
     end()
 
+
 def main(args):
     "Run all regression tests."
 
     # Check command-line arguments TODO: Use argparse
-    generate_only  = "--generate-only" in args
-    fast           = "--fast" in args
-    bench          = "--bench" in args
-    use_quad       = "--skip-quad" not in args
-    use_ext_quad   = "--ext-quad" in args
-    use_ext_uflacs = "--ext-uflacs" in args
-    permissive     = "--permissive" in args
-    tolerant       = "--tolerant" in args
-    print_timing   = "--print-timing" in args
-    skip_download  = "--skip-download" in args
-    ignore_code_diff = "--ignore-code-diff" in args
+    use_auto = "--skip-auto" not in args
+    use_uflacs = "--skip-uflacs" not in args
+    use_quad = "--skip-quad" not in args
+    use_ext_quad = "--ext-quad" in args
+
+    skip_download = "--skip-download" in args
+    skip_run = "--skip-run" in args
+    skip_code_diff = "--skip-code-diff" in args
+    skip_validate = "--skip-validate" in args
+    bench = "--bench" in args
+
+    permissive = "--permissive" in args
+    tolerant = "--tolerant" in args
+    print_timing = "--print-timing" in args
+    show_help = "--help" in args
 
     flags = (
-        "--generate-only",
-        "--fast",
-        "--bench",
+        "--skip-auto",
+        "--skip-uflacs",
         "--skip-quad",
         "--ext-quad",
-        "--ext-uflacs",
+        "--skip-download",
+        "--skip-run",
+        "--skip-code-diff",
+        "--skip-validate",
+        "--bench",
         "--permissive",
         "--tolerant",
         "--print-timing",
-        "--skip-download",
-        "--ignore-code-diff",
-        )
-    args = [arg for arg in args if not arg in flags]
+        "--help",
+    )
+    args = [arg for arg in args if arg not in flags]
+
+    if show_help:
+        info("Valid arguments:\n" + "\n".join(flags))
+        return 0
+
+    if bench or not skip_validate:
+        skip_run = False
+    if bench:
+        skip_code_diff = True
+        skip_validate = True
 
     # Extract .ufl names from args
     only_forms = set([arg for arg in args if arg.endswith(".ufl")])
@@ -409,12 +514,14 @@ def main(args):
     if skip_download:
         info_blue("Skipping reference data download")
     else:
-        failure, output = get_status_output("./scripts/download")
-        print(output)
-        if failure:
-            info_red("Download reference data failed")
-        else:
+        try:
+            cmd = "./scripts/download"
+            output = as_native_str(subprocess.check_output(cmd, shell=True))
+            print(output)
             info_green("Download reference data ok")
+        except subprocess.CalledProcessError as e:
+            print(e.output)
+            info_red("Download reference data failed")
 
     if tolerant:
         global output_tolerance
@@ -425,19 +532,21 @@ def main(args):
     clean_output(output_directory)
     os.chdir(output_directory)
 
-    # Adjust which test cases (combinations of compile arguments) to
-    # run here
-    test_cases = ["-r auto"]
-    if use_quad and (not bench and not fast):
+    # Adjust which test cases (combinations of compile arguments) to run here
+    test_cases = []
+    if use_auto:
+        test_cases += ["-r auto"]
+    if use_uflacs:
+        test_cases += ["-r uflacs"]
+    if use_quad:
         test_cases += ["-r quadrature", "-r quadrature -O"]
     if use_ext_quad:
         test_cases += ext_quad
-    if use_ext_uflacs:
-        test_cases = ext_uflacs
-        test_cases += ["-r quadrature"]
-        #test_cases += ["-r quadrature -O"]
+
+    test_case_timings = {}
 
     for argument in test_cases:
+        test_case_timings[argument] = time.time()
 
         begin("Running regression tests with %s" % argument)
 
@@ -446,14 +555,21 @@ def main(args):
         clean_output(sub_directory)
         os.chdir(sub_directory)
 
+        # Workarounds for partial feature completeness in uflacs
+        if "uflacs" in argument and not only_forms:
+            skip_forms = known_uflacs_failures
+            info_blue("Skipping forms known to fail with uflacs:\n" + "\n".join(sorted(skip_forms)))
+        else:
+            skip_forms = set()
+
         # Generate test cases
-        generate_test_cases(bench, only_forms)
+        generate_test_cases(bench, only_forms, skip_forms)
 
         # Generate code
-        generate_code(args + [argument], only_forms)
+        generate_code(args + argument.split(), only_forms, skip_forms)
 
         # Location of reference directories
-        reference_directory =  os.path.abspath("../../ffc-reference-data/")
+        reference_directory = os.path.abspath("../../ffc-reference-data/")
         code_reference_dir = os.path.join(reference_directory, sub_directory)
 
         # Note: We use the r_auto references for all test cases. This
@@ -463,38 +579,51 @@ def main(args):
 
         # Validate code by comparing to code generated with this set
         # of compiler parameters
-        if not bench and (argument not in ext_quad) and not ignore_code_diff:
+        if skip_code_diff or (argument in ext_quad):
+            info_blue("Skipping code diff validation")
+        else:
             validate_code(code_reference_dir)
 
         # Build and run programs and validate output to common
         # reference
-        if fast or generate_only:
-            info("Skipping program validation")
-        elif bench:
-            build_programs(bench, permissive)
-            run_programs(bench)
+        if skip_run:
+            info_blue("Skipping program execution")
         else:
             build_programs(bench, permissive)
             run_programs(bench)
-            validate_programs(output_reference_dir)
+            # Validate output to common reference results
+            if skip_validate:
+                info_blue("Skipping program output validation")
+            else:
+                validate_programs(output_reference_dir)
 
         # Go back up
         os.chdir(os.path.pardir)
 
         end()
+        test_case_timings[argument] = time.time() - test_case_timings[argument]
+
+    # Go back up
+    os.chdir(os.path.pardir)
 
     # Print results
     if print_timing:
-        timings = '\n'.join("%10.2e s  %s" % (t, name) for (name, t) in _command_timings)
         info_green("Timing of all commands executed:")
-        info(timings)
-    if logfile is None:
+        timings = '\n'.join("%10.2e s  %s" % (t, name) for (name, t)
+                            in _command_timings)
+        info_blue(timings)
+
+    for argument in test_cases:
+        info_blue("Total time for %s: %.1f s" % (argument, test_case_timings[argument]))
+
+    if not os.path.isfile(logfile) or os.stat(logfile).st_size == 0:
         info_green("Regression tests OK")
         return 0
     else:
         info_red("Regression tests failed")
-        info("Error messages stored in error.log")
+        info_red("Error messages stored in %s" % logfile)
         return 1
 
+
 if __name__ == "__main__":
     sys.exit(main(sys.argv[1:]))
